/* ex: set ro ft=c:
 * !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
 *
 * This file is generated automatically from './src/pmc/complex.pmc'
 * by tools/build/pmc2c.pl.
 *
 * Any changes made here will be lost!
 *
 */

/* HEADERIZER HFILE: none */
/* HEADERIZER STOP */

#include "parrot/parrot.h"
#include "parrot/extend.h"
#include "parrot/dynext.h"
#include "pmc_fixedintegerarray.h"
#include "pmc_complex.h"
#include "pmc_default.h"
#include "complex.str"
#include "pmc/pmc_continuation.h"
#include "pmc/pmc_callcontext.h"
#line 1 "./src/pmc/complex.pmc"
/*
Copyright (C) 2004-2009, Parrot Foundation.
$Id: complex.pmc 44779 2010-03-08 23:09:26Z plobsing $

=head1 NAME

src/pmc/complex.pmc - Complex Numbers PMC Class

=head1 DESCRIPTION

C<Complex> provides a representation of complex numbers. It handles
string parsing/generating and basic mathematical operations.

=head2 Functions

=over 4

=cut

Equations used are sometimes listed.  At times, multiple equations are given,
but those starting with => are the ones used

*/

/*

=item C<static void
complex_parse_string(PARROT_INTERP, FLOATVAL *re, FLOATVAL *im, STRING *value)>

Parses the string in C<value> to produce a complex number, represented
by the real (C<*re>) and imaginary (C<*im>) parts. Raises an exception
if it cannot understand the string.  The string should be of the form
C<a+bi> with optional spaces around C<+> and before C<i>. You can also
use C<j> instead of C<i>.

=cut

*/

static void
complex_parse_string(PARROT_INTERP, FLOATVAL *re, FLOATVAL *im, STRING *value) {
    char   * const str        = Parrot_str_to_cstring(interp, value);
    char   *t                 = str;
    char   *first_num_offset  = str;
    char   *second_num_offset = NULL;
    STRING *S;

    INTVAL  i                 = 0;
    INTVAL  first_num_minus   = 0;
    INTVAL  second_num_minus  = 0;
    UINTVAL first_num_length, second_num_length;

    /* walk the string and identify the real and imaginary parts */

    if (*t == '-') {
        /* first number is negative */
        t++;
        first_num_minus = 1;

        /* allow for an optional space */
        if (*t == ' ')
            t++;
        first_num_offset = t;
    }

    /* skip digits */
    while (*t >= '0' && *t <= '9')
        t++;

    if (*t == '.') {
        /* this number has a decimal point */
        t++;

        /* skip digits */
        while (*t >= '0' && *t <= '9')
            t++;
    }

    /* save the length of the real part */
    first_num_length = t - first_num_offset;

    /* end of string; we only have a real part */
    if (*t == 0) {
        second_num_length = 0;
    }
    else if ((*t == 'i' || *t == 'j') && *(t+1) == 0) {
        /* there is only an imaginary part, so the first number was
            actually the imaginary part */
        second_num_length = first_num_length;
        first_num_length  = 0;
        second_num_offset = first_num_offset;
        second_num_minus  = first_num_minus;
        first_num_minus   = 0;

        /* this is useful if there is no number for
            the imaginary part, like in "-i" */
        i = 1;
    }
    else {
        /* skip an optional space */
        if (*t == ' ')
            t++;

        /* expect "+" or "-" and the imaginary part */
        if (*t == '+' || *t == '-') {
            /* save the sign */
            second_num_minus = (*t == '-');
            t++;

            /* skip another optional space */
            if (*t == ' ')
                t++;

            /* save the beginning of the imaginary part */
            second_num_offset = t;

            /* skip digits */
            while (*t >= '0' && *t <= '9')
                t++;

            if (*t == '.') {
                /* this number has a decimal point */
                t++;

                /* skip digits */
                while (*t >= '0' && *t <= '9')
                    t++;
            }

            /* save the length of the imaginary part */
            second_num_length = t - second_num_offset;

            /* allow for one more optional space */
            if (*t == ' ')
                t++;

            /* verify that the string ends properly */
            if ((*t != 'i' && *t != 'j') || (*(t+1) != 0)) {
                /* imaginary part does not end in 'i' or 'j' */
                Parrot_ex_throw_from_c_args(interp, NULL,
                    EXCEPTION_INVALID_STRING_REPRESENTATION,
                    "Complex: malformed string");
            }

            /* this is useful if there is no number for the
                imaginary part, like in "2+i" */
            i = 1;

            /* all is OK, save the number */
        }
        else {
            /* "+" or "-" not found: error */
            Parrot_str_free_cstring(str);

            Parrot_ex_throw_from_c_args(interp, NULL,
                EXCEPTION_INVALID_STRING_REPRESENTATION,
                "Complex: malformed string");
        }
    }

    /* now we have the offsets and the lengths we turn them into float values */

    if (first_num_length) {
        /* there is a real part, interpret it */
        S   = Parrot_str_new(interp, first_num_offset, first_num_length);
        *re = Parrot_str_to_num(interp, S);
    }
    else {
        /* consider the real part 0.0 */
        *re = 0.0;
    }

    if (second_num_length) {
        /* there is an imaginary part, interpret it */
        S   = Parrot_str_new(interp, second_num_offset, second_num_length);
        *im = Parrot_str_to_num(interp, S);
    }
    else {
        /* consider the imaginary part 0.0 */
        if (i) /* the string was something like "1+i" */
            *im = 1.0;
        else
            *im = 0.0;
    }

    if (first_num_minus)
        *re = -*re;

    if (second_num_minus)
        *im = -*im;

    Parrot_str_free_cstring(str);
}

static void
int_check_divide_zero(PARROT_INTERP, INTVAL value) {
    if (value == 0)
        Parrot_ex_throw_from_c_args(interp, NULL, EXCEPTION_DIV_BY_ZERO,
            "Divide by zero");
}

static void
float_check_divide_zero(PARROT_INTERP, FLOATVAL value) {
    if (FLOAT_IS_ZERO(value))
        Parrot_ex_throw_from_c_args(interp, NULL, EXCEPTION_DIV_BY_ZERO,
            "Divide by zero");
}

static void
complex_check_divide_zero(PARROT_INTERP, PMC *value) {
    /* Throw an exception if we are dividing by zero. Check both the real part
     * and the imaginary part.*/
    if (FLOAT_IS_ZERO(VTABLE_get_number_keyed_int(interp, value, 0))
            && FLOAT_IS_ZERO(VTABLE_get_number_keyed_int(interp, value, 1)))
        Parrot_ex_throw_from_c_args(interp, NULL, EXCEPTION_DIV_BY_ZERO,
            "Divide by zero");
}
#line 241 "./src/pmc/complex.c"
static  PMC  * Parrot_Complex_absolute(PARROT_INTERP, PMC *pmc, PMC *dest);
static  PMC * Parrot_Complex_add(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  PMC  * Parrot_Complex_add_float(PARROT_INTERP, PMC *pmc, FLOATVAL value, PMC *dest);
static  PMC  * Parrot_Complex_add_int(PARROT_INTERP, PMC *pmc, INTVAL value, PMC *dest);
static  PMC  * Parrot_Complex_clone(PARROT_INTERP, PMC *pmc);
static  PMC * Parrot_Complex_divide(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  PMC  * Parrot_Complex_divide_float(PARROT_INTERP, PMC *pmc, FLOATVAL value, PMC *dest);
static  PMC  * Parrot_Complex_divide_int(PARROT_INTERP, PMC *pmc, INTVAL value, PMC *dest);
static  void  Parrot_Complex_freeze(PARROT_INTERP, PMC *pmc, PMC *visit);
static  INTVAL  Parrot_Complex_get_bool(PARROT_INTERP, PMC *pmc);
static  INTVAL  Parrot_Complex_get_integer(PARROT_INTERP, PMC *pmc);
static  INTVAL  Parrot_Complex_get_integer_keyed(PARROT_INTERP, PMC *pmc, PMC *key);
static  INTVAL  Parrot_Complex_get_integer_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key);
static  FLOATVAL  Parrot_Complex_get_number(PARROT_INTERP, PMC *pmc);
static  FLOATVAL  Parrot_Complex_get_number_keyed(PARROT_INTERP, PMC *pmc, PMC *key);
static  FLOATVAL  Parrot_Complex_get_number_keyed_int(PARROT_INTERP, PMC *pmc, INTVAL key);
static  FLOATVAL  Parrot_Complex_get_number_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key);
static  PMC  * Parrot_Complex_get_pmc_keyed(PARROT_INTERP, PMC *pmc, PMC *key);
static  PMC  * Parrot_Complex_get_pmc_keyed_int(PARROT_INTERP, PMC *pmc, INTVAL key);
static  PMC  * Parrot_Complex_get_pmc_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key);
static  STRING  * Parrot_Complex_get_string(PARROT_INTERP, PMC *pmc);
static  void  Parrot_Complex_i_absolute(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_i_add(PARROT_INTERP, PMC *pmc, PMC *value);
static  void  Parrot_Complex_i_add_float(PARROT_INTERP, PMC *pmc, FLOATVAL value);
static  void  Parrot_Complex_i_add_int(PARROT_INTERP, PMC *pmc, INTVAL value);
static  void Parrot_Complex_i_divide(PARROT_INTERP, PMC *pmc, PMC *value);
static  void  Parrot_Complex_i_divide_float(PARROT_INTERP, PMC *pmc, FLOATVAL value);
static  void  Parrot_Complex_i_divide_int(PARROT_INTERP, PMC *pmc, INTVAL value);
static  void Parrot_Complex_i_multiply(PARROT_INTERP, PMC *pmc, PMC *value);
static  void  Parrot_Complex_i_multiply_float(PARROT_INTERP, PMC *pmc, FLOATVAL value);
static  void  Parrot_Complex_i_multiply_int(PARROT_INTERP, PMC *pmc, INTVAL value);
static  void  Parrot_Complex_i_neg(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_i_subtract(PARROT_INTERP, PMC *pmc, PMC *value);
static  void  Parrot_Complex_i_subtract_float(PARROT_INTERP, PMC *pmc, FLOATVAL value);
static  void  Parrot_Complex_i_subtract_int(PARROT_INTERP, PMC *pmc, INTVAL value);
static  void  Parrot_Complex_init(PARROT_INTERP, PMC *pmc);
static  void  Parrot_Complex_init_pmc(PARROT_INTERP, PMC *pmc, PMC *initializer);
static  INTVAL Parrot_Complex_is_equal(PARROT_INTERP, PMC *pmc, PMC *value);
static  PMC * Parrot_Complex_multiply(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  PMC  * Parrot_Complex_multiply_float(PARROT_INTERP, PMC *pmc, FLOATVAL value, PMC *dest);
static  PMC  * Parrot_Complex_multiply_int(PARROT_INTERP, PMC *pmc, INTVAL value, PMC *dest);
static  PMC  * Parrot_Complex_neg(PARROT_INTERP, PMC *pmc, PMC *dest);
static  void  Parrot_Complex_set_integer_keyed(PARROT_INTERP, PMC *pmc, PMC *key, INTVAL value);
static  void  Parrot_Complex_set_integer_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key, INTVAL value);
static  void  Parrot_Complex_set_integer_native(PARROT_INTERP, PMC *pmc, INTVAL value);
static  void  Parrot_Complex_set_number_keyed(PARROT_INTERP, PMC *pmc, PMC *key, FLOATVAL value);
static  void  Parrot_Complex_set_number_keyed_int(PARROT_INTERP, PMC *pmc, INTVAL key, FLOATVAL v);
static  void  Parrot_Complex_set_number_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key, FLOATVAL value);
static  void  Parrot_Complex_set_number_native(PARROT_INTERP, PMC *pmc, FLOATVAL value);
static  void  Parrot_Complex_set_pmc(PARROT_INTERP, PMC *pmc, PMC *value);
static  void  Parrot_Complex_set_pmc_keyed(PARROT_INTERP, PMC *pmc, PMC *key, PMC *value);
static  void  Parrot_Complex_set_pmc_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key, PMC *value);
static  void  Parrot_Complex_set_string_native(PARROT_INTERP, PMC *pmc, STRING *value);
static  PMC * Parrot_Complex_subtract(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  PMC  * Parrot_Complex_subtract_float(PARROT_INTERP, PMC *pmc, FLOATVAL value, PMC *dest);
static  PMC  * Parrot_Complex_subtract_int(PARROT_INTERP, PMC *pmc, INTVAL value, PMC *dest);
static  void  Parrot_Complex_thaw(PARROT_INTERP, PMC *pmc, PMC *visit);
static  PMC  * Parrot_Complex_multi_add_Complex_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  PMC  * Parrot_Complex_multi_add_DEFAULT_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  void  Parrot_Complex_multi_i_add_Complex(PARROT_INTERP, PMC *pmc, PMC *value);
static  void  Parrot_Complex_multi_i_add_DEFAULT(PARROT_INTERP, PMC *pmc, PMC *value);
static  PMC  * Parrot_Complex_multi_subtract_Complex_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  PMC  * Parrot_Complex_multi_subtract_DEFAULT_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  void  Parrot_Complex_multi_i_subtract_Complex(PARROT_INTERP, PMC *pmc, PMC *value);
static  void  Parrot_Complex_multi_i_subtract_DEFAULT(PARROT_INTERP, PMC *pmc, PMC *value);
static  PMC  * Parrot_Complex_multi_multiply_Complex_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  PMC  * Parrot_Complex_multi_multiply_DEFAULT_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  void  Parrot_Complex_multi_i_multiply_Complex(PARROT_INTERP, PMC *pmc, PMC *value);
static  void  Parrot_Complex_multi_i_multiply_DEFAULT(PARROT_INTERP, PMC *pmc, PMC *value);
static  PMC  * Parrot_Complex_multi_divide_Complex_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  PMC  * Parrot_Complex_multi_divide_DEFAULT_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  void  Parrot_Complex_multi_i_divide_Complex(PARROT_INTERP, PMC *pmc, PMC *value);
static  void  Parrot_Complex_multi_i_divide_DEFAULT(PARROT_INTERP, PMC *pmc, PMC *value);
static  INTVAL  Parrot_Complex_multi_is_equal_Complex(PARROT_INTERP, PMC *pmc, PMC *value);
static  INTVAL  Parrot_Complex_multi_is_equal_DEFAULT(PARROT_INTERP, PMC *pmc, PMC *value);
static  void Parrot_Complex_nci_ln(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_exp(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_sin(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_cos(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_tan(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_cot(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_sec(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_csc(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_asin(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_acos(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_atan(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_acot(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_acsc(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_asec(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_sinh(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_cosh(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_tanh(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_coth(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_csch(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_sech(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_asinh(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_acosh(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_atanh(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_acoth(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_acsch(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_asech(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_pow(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_sqrt(PARROT_INTERP, PMC *pmc);
void    Parrot_Complex_class_init(PARROT_INTERP, int, int);
PARROT_EXPORT VTABLE* Parrot_Complex_update_vtable(ARGMOD(VTABLE*));
PARROT_EXPORT VTABLE* Parrot_Complex_get_vtable(PARROT_INTERP);
void    Parrot_Complex_ro_class_init(PARROT_INTERP, int, int);
PARROT_EXPORT VTABLE* Parrot_Complex_ro_update_vtable(ARGMOD(VTABLE*));
PARROT_EXPORT VTABLE* Parrot_Complex_ro_get_vtable(PARROT_INTERP);
static  PMC  *
Parrot_Complex_absolute(PARROT_INTERP, PMC *pmc, PMC *dest)
{
#line 1166 "./src/pmc/complex.pmc"

    FLOATVAL re, im, d;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    d = sqrt(re*re + im*im);

    dest = Parrot_pmc_new(interp,
        Parrot_get_ctx_HLL_type(interp, enum_class_Float));

    VTABLE_set_number_native(interp, dest, d);
    return dest;
#line 366 "./src/pmc/complex.c"
}
static  PMC *
Parrot_Complex_add(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
    INTVAL type = VTABLE_type(interp, value);
    /* For dynpmc fallback to MMD */
    if ((type >= enum_class_core_max) || (VTABLE_type(interp, pmc) >= enum_class_core_max))
        type = enum_class_core_max;
    switch(type) {
        case enum_class_Complex:
            return Parrot_Complex_multi_add_Complex_PMC(interp, pmc, value, dest);
            break;

        default:
            if (type < enum_class_core_max)
                return Parrot_Complex_multi_add_DEFAULT_PMC(interp, pmc, value, dest);
            else {
                PMC *retval = PMCNULL;
                Parrot_mmd_multi_dispatch_from_c_args(interp, "add", "PPP->P", pmc, value, dest, &retval);
                return retval;
            }
            break;

    }
}
static  PMC  *
Parrot_Complex_add_float(PARROT_INTERP, PMC *pmc, FLOATVAL value, PMC *dest)
{
#line 673 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    dest = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, dest, re + value);
    SETATTR_Complex_im(interp, dest, im);

    return dest;
#line 406 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_add_int(PARROT_INTERP, PMC *pmc, INTVAL value, PMC *dest)
{
#line 669 "./src/pmc/complex.pmc"

    return VTABLE_add_float(interp, pmc, (FLOATVAL)value, dest);
#line 414 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_clone(PARROT_INTERP, PMC *pmc)
{
#line 287 "./src/pmc/complex.pmc"

    PMC * const dest = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));
    FLOATVAL re, im;

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, dest, re);
    SETATTR_Complex_im(interp, dest, im);

    return dest;
#line 430 "./src/pmc/complex.c"
}
static  PMC *
Parrot_Complex_divide(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
    INTVAL type = VTABLE_type(interp, value);
    /* For dynpmc fallback to MMD */
    if ((type >= enum_class_core_max) || (VTABLE_type(interp, pmc) >= enum_class_core_max))
        type = enum_class_core_max;
    switch(type) {
        case enum_class_Complex:
            return Parrot_Complex_multi_divide_Complex_PMC(interp, pmc, value, dest);
            break;

        default:
            if (type < enum_class_core_max)
                return Parrot_Complex_multi_divide_DEFAULT_PMC(interp, pmc, value, dest);
            else {
                PMC *retval = PMCNULL;
                Parrot_mmd_multi_dispatch_from_c_args(interp, "divide", "PPP->P", pmc, value, dest, &retval);
                return retval;
            }
            break;

    }
}
static  PMC  *
Parrot_Complex_divide_float(PARROT_INTERP, PMC *pmc, FLOATVAL value, PMC *dest)
{
#line 1003 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    float_check_divide_zero(interp, value);
    dest = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, dest, re / value);
    SETATTR_Complex_im(interp, dest, im / value);

    return dest;
#line 472 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_divide_int(PARROT_INTERP, PMC *pmc, INTVAL value, PMC *dest)
{
#line 989 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    int_check_divide_zero(interp, value);
    dest = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, dest, re / value);
    SETATTR_Complex_im(interp, dest, im / value);

    return dest;
#line 490 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_freeze(PARROT_INTERP, PMC *pmc, PMC *visit)
{
#line 311 "./src/pmc/complex.pmc"

    FLOATVAL re, im;

    interp->vtables[enum_class_default]->freeze(interp, pmc, visit);

    GETATTR_Complex_re(interp, pmc, re);
    VTABLE_push_float(interp, visit, re);

    GETATTR_Complex_im(interp, pmc, im);
    VTABLE_push_float(interp, visit, im);
#line 506 "./src/pmc/complex.c"
}
static  INTVAL 
Parrot_Complex_get_bool(PARROT_INTERP, PMC *pmc)
{
#line 376 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    return !(FLOAT_IS_ZERO(re) && FLOAT_IS_ZERO(im));
#line 517 "./src/pmc/complex.c"
}
static  INTVAL 
Parrot_Complex_get_integer(PARROT_INTERP, PMC *pmc)
{
#line 357 "./src/pmc/complex.pmc"

    const FLOATVAL f = VTABLE_get_number(interp, pmc);
    return (INTVAL)f;
#line 526 "./src/pmc/complex.c"
}
static  INTVAL 
Parrot_Complex_get_integer_keyed(PARROT_INTERP, PMC *pmc, PMC *key)
{
#line 403 "./src/pmc/complex.pmc"

    STRING * const s = VTABLE_get_string(interp, key);
    return VTABLE_get_integer_keyed_str(interp, pmc, s);
#line 535 "./src/pmc/complex.c"
}
static  INTVAL 
Parrot_Complex_get_integer_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key)
{
#line 408 "./src/pmc/complex.pmc"

    const FLOATVAL f = VTABLE_get_number_keyed_str(interp, pmc, key);
    return (INTVAL)f;
#line 544 "./src/pmc/complex.c"
}
static  FLOATVAL 
Parrot_Complex_get_number(PARROT_INTERP, PMC *pmc)
{
#line 362 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    return sqrt(re * re + im * im);
#line 555 "./src/pmc/complex.c"
}
static  FLOATVAL 
Parrot_Complex_get_number_keyed(PARROT_INTERP, PMC *pmc, PMC *key)
{
#line 413 "./src/pmc/complex.pmc"

    STRING * const s = VTABLE_get_string(interp, key);
    return VTABLE_get_number_keyed_str(interp, pmc, s);
#line 564 "./src/pmc/complex.c"
}
static  FLOATVAL 
Parrot_Complex_get_number_keyed_int(PARROT_INTERP, PMC *pmc, INTVAL key)
{
#line 484 "./src/pmc/complex.pmc"

    FLOATVAL f;
    switch (key) {
      case 0:
        GETATTR_Complex_re(interp, pmc, f);
        break;
      case 1:
        GETATTR_Complex_im(interp, pmc, f);
        break;
      default:
        Parrot_ex_throw_from_c_args(interp, NULL,
                EXCEPTION_INVALID_OPERATION, "Complex: key must be 0 or 1");
    }
    return f;
#line 584 "./src/pmc/complex.c"
}
static  FLOATVAL 
Parrot_Complex_get_number_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key)
{
#line 418 "./src/pmc/complex.pmc"

    FLOATVAL value;
    if (Parrot_str_equal(interp, key, CONST_STRING(interp, "real"))) {
        GETATTR_Complex_re(interp, pmc, value);
    }
    else if (Parrot_str_equal(interp, key, CONST_STRING(interp, "imag"))) {
        GETATTR_Complex_im(interp, pmc, value);
    }
    else
        Parrot_ex_throw_from_c_args(interp, NULL,
            EXCEPTION_INVALID_OPERATION, "Complex: key is neither 'real' or 'imag'");
    return value;
#line 602 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_get_pmc_keyed(PARROT_INTERP, PMC *pmc, PMC *key)
{
#line 432 "./src/pmc/complex.pmc"

    if (VTABLE_isa(interp, key, CONST_STRING(interp, "Integer"))) {
        const INTVAL i = VTABLE_get_integer(interp, key);
        return VTABLE_get_pmc_keyed_int(interp, pmc, i);
    }
    else {
        STRING * const s = VTABLE_get_string(interp, key);
        return VTABLE_get_pmc_keyed_str(interp, pmc, s);
    }
#line 617 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_get_pmc_keyed_int(PARROT_INTERP, PMC *pmc, INTVAL key)
{
#line 460 "./src/pmc/complex.pmc"

    PMC * const    ret = Parrot_pmc_new(interp, enum_class_Float);
    const FLOATVAL val = VTABLE_get_number_keyed_int(interp, pmc, key);
    VTABLE_set_number_native(interp, ret, val);
    return ret;
#line 628 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_get_pmc_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key)
{
#line 443 "./src/pmc/complex.pmc"

    PMC * const    ret = Parrot_pmc_new(interp, enum_class_Float);
    const FLOATVAL val = VTABLE_get_number_keyed_str(interp, pmc, key);
    VTABLE_set_number_native(interp, ret, val);
    return ret;
#line 639 "./src/pmc/complex.c"
}
static  STRING  *
Parrot_Complex_get_string(PARROT_INTERP, PMC *pmc)
{
#line 369 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    return Parrot_sprintf_c(interp, "%vg%+vgi", re, im);
#line 650 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_i_absolute(PARROT_INTERP, PMC *pmc)
{
#line 1179 "./src/pmc/complex.pmc"

    FLOATVAL re, im, d;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    d = sqrt(re*re + im*im);
    Parrot_pmc_reuse(interp, pmc, enum_class_Float, 0);
    VTABLE_set_number_native(interp, pmc, d);
#line 663 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_i_add(PARROT_INTERP, PMC *pmc, PMC *value)
{
    INTVAL type = VTABLE_type(interp, value);
    /* For dynpmc fallback to MMD */
    if ((type >= enum_class_core_max) || (VTABLE_type(interp, pmc) >= enum_class_core_max))
        type = enum_class_core_max;
    switch(type) {
        case enum_class_Complex:
            Parrot_Complex_multi_i_add_Complex(interp, pmc, value);
            break;

        default:
            if (type < enum_class_core_max)
                Parrot_Complex_multi_i_add_DEFAULT(interp, pmc, value);
            else {
                
                Parrot_mmd_multi_dispatch_from_c_args(interp, "i_add", "PP->", pmc, value);
                return;
            }
            break;

    }
}
static  void 
Parrot_Complex_i_add_float(PARROT_INTERP, PMC *pmc, FLOATVAL value)
{
#line 709 "./src/pmc/complex.pmc"

    FLOATVAL re;

    GETATTR_Complex_re(interp, pmc, re);
    SETATTR_Complex_re(interp, pmc, re + value);
#line 698 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_i_add_int(PARROT_INTERP, PMC *pmc, INTVAL value)
{
#line 705 "./src/pmc/complex.pmc"

    VTABLE_i_add_float(interp, pmc, (FLOATVAL)value);
#line 706 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_i_divide(PARROT_INTERP, PMC *pmc, PMC *value)
{
    INTVAL type = VTABLE_type(interp, value);
    /* For dynpmc fallback to MMD */
    if ((type >= enum_class_core_max) || (VTABLE_type(interp, pmc) >= enum_class_core_max))
        type = enum_class_core_max;
    switch(type) {
        case enum_class_Complex:
            Parrot_Complex_multi_i_divide_Complex(interp, pmc, value);
            break;

        default:
            if (type < enum_class_core_max)
                Parrot_Complex_multi_i_divide_DEFAULT(interp, pmc, value);
            else {
                
                Parrot_mmd_multi_dispatch_from_c_args(interp, "i_divide", "PP->", pmc, value);
                return;
            }
            break;

    }
}
static  void 
Parrot_Complex_i_divide_float(PARROT_INTERP, PMC *pmc, FLOATVAL value)
{
#line 1068 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    float_check_divide_zero(interp, value);

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, pmc, re / value);
    SETATTR_Complex_im(interp, pmc, im / value);
#line 745 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_i_divide_int(PARROT_INTERP, PMC *pmc, INTVAL value)
{
#line 1057 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    int_check_divide_zero(interp, value);

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, pmc, re / value);
    SETATTR_Complex_im(interp, pmc, im / value);
#line 760 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_i_multiply(PARROT_INTERP, PMC *pmc, PMC *value)
{
    INTVAL type = VTABLE_type(interp, value);
    /* For dynpmc fallback to MMD */
    if ((type >= enum_class_core_max) || (VTABLE_type(interp, pmc) >= enum_class_core_max))
        type = enum_class_core_max;
    switch(type) {
        case enum_class_Complex:
            Parrot_Complex_multi_i_multiply_Complex(interp, pmc, value);
            break;

        default:
            if (type < enum_class_core_max)
                Parrot_Complex_multi_i_multiply_DEFAULT(interp, pmc, value);
            else {
                
                Parrot_mmd_multi_dispatch_from_c_args(interp, "i_multiply", "PP->", pmc, value);
                return;
            }
            break;

    }
}
static  void 
Parrot_Complex_i_multiply_float(PARROT_INTERP, PMC *pmc, FLOATVAL value)
{
#line 904 "./src/pmc/complex.pmc"

    FLOATVAL re, im;

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, pmc, re * value);
    SETATTR_Complex_im(interp, pmc, im * value);
#line 797 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_i_multiply_int(PARROT_INTERP, PMC *pmc, INTVAL value)
{
#line 895 "./src/pmc/complex.pmc"

    FLOATVAL re, im;

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, pmc, re * value);
    SETATTR_Complex_im(interp, pmc, im * value);
#line 810 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_i_neg(PARROT_INTERP, PMC *pmc)
{
#line 1104 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, pmc, -re);
    SETATTR_Complex_im(interp, pmc, -im);
#line 822 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_i_subtract(PARROT_INTERP, PMC *pmc, PMC *value)
{
    INTVAL type = VTABLE_type(interp, value);
    /* For dynpmc fallback to MMD */
    if ((type >= enum_class_core_max) || (VTABLE_type(interp, pmc) >= enum_class_core_max))
        type = enum_class_core_max;
    switch(type) {
        case enum_class_Complex:
            Parrot_Complex_multi_i_subtract_Complex(interp, pmc, value);
            break;

        default:
            if (type < enum_class_core_max)
                Parrot_Complex_multi_i_subtract_DEFAULT(interp, pmc, value);
            else {
                
                Parrot_mmd_multi_dispatch_from_c_args(interp, "i_subtract", "PP->", pmc, value);
                return;
            }
            break;

    }
}
static  void 
Parrot_Complex_i_subtract_float(PARROT_INTERP, PMC *pmc, FLOATVAL value)
{
#line 798 "./src/pmc/complex.pmc"

    FLOATVAL re;

    GETATTR_Complex_re(interp, pmc, re);
    SETATTR_Complex_re(interp, pmc, re - value);
#line 857 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_i_subtract_int(PARROT_INTERP, PMC *pmc, INTVAL value)
{
#line 794 "./src/pmc/complex.pmc"

    VTABLE_i_subtract_float(interp, pmc, (FLOATVAL) value);
#line 865 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_init(PARROT_INTERP, PMC *pmc)
{
#line 250 "./src/pmc/complex.pmc"

    SETATTR_Complex_re(interp, pmc, 0.0);
    SETATTR_Complex_im(interp, pmc, 0.0);
#line 874 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_init_pmc(PARROT_INTERP, PMC *pmc, PMC *initializer)
{
#line 255 "./src/pmc/complex.pmc"

    const INTVAL arg_type = VTABLE_type(interp, initializer);
    VTABLE_init(interp, pmc);
    switch (arg_type) {
      case enum_class_String:
        VTABLE_set_string_native(interp, pmc, VTABLE_get_string(interp, initializer));
        break;
      case enum_class_FixedFloatArray:
      case enum_class_ResizableFloatArray:
      case enum_class_FixedIntegerArray:
      case enum_class_ResizableIntegerArray:
        if (VTABLE_get_integer(interp, initializer) == 2) {
            FLOATVAL re = VTABLE_get_number_keyed_int(interp, initializer, 0);
            FLOATVAL im = VTABLE_get_number_keyed_int(interp, initializer, 1);
            SETATTR_Complex_re(interp, pmc, re);
            SETATTR_Complex_im(interp, pmc, im);
            break;
        }
        /* else let it fall to default */
      default:
        if (VTABLE_isa(interp, initializer, CONST_STRING(interp, "String"))) {
            STRING * s = VTABLE_get_string(interp, initializer);
            VTABLE_set_string_native(interp, pmc, s);
        }
        else {
            Parrot_ex_throw_from_c_args(interp, NULL,
                    EXCEPTION_INVALID_OPERATION,
                    "Invalid Complex initializer");
        }
    }
#line 910 "./src/pmc/complex.c"
}
static  INTVAL
Parrot_Complex_is_equal(PARROT_INTERP, PMC *pmc, PMC *value)
{
    INTVAL type = VTABLE_type(interp, value);
    /* For dynpmc fallback to MMD */
    if ((type >= enum_class_core_max) || (VTABLE_type(interp, pmc) >= enum_class_core_max))
        type = enum_class_core_max;
    switch(type) {
        case enum_class_Complex:
            return Parrot_Complex_multi_is_equal_Complex(interp, pmc, value);
            break;

        default:
            if (type < enum_class_core_max)
                return Parrot_Complex_multi_is_equal_DEFAULT(interp, pmc, value);
            else {
                INTVAL retval;
                Parrot_mmd_multi_dispatch_from_c_args(interp, "is_equal", "PP->I", pmc, value, &retval);
                return retval;
            }
            break;

    }
}
static  PMC *
Parrot_Complex_multiply(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
    INTVAL type = VTABLE_type(interp, value);
    /* For dynpmc fallback to MMD */
    if ((type >= enum_class_core_max) || (VTABLE_type(interp, pmc) >= enum_class_core_max))
        type = enum_class_core_max;
    switch(type) {
        case enum_class_Complex:
            return Parrot_Complex_multi_multiply_Complex_PMC(interp, pmc, value, dest);
            break;

        default:
            if (type < enum_class_core_max)
                return Parrot_Complex_multi_multiply_DEFAULT_PMC(interp, pmc, value, dest);
            else {
                PMC *retval = PMCNULL;
                Parrot_mmd_multi_dispatch_from_c_args(interp, "multiply", "PPP->P", pmc, value, dest, &retval);
                return retval;
            }
            break;

    }
}
static  PMC  *
Parrot_Complex_multiply_float(PARROT_INTERP, PMC *pmc, FLOATVAL value, PMC *dest)
{
#line 863 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    dest = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, dest, re * value);
    SETATTR_Complex_im(interp, dest, im * value);

    return dest;
#line 974 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_multiply_int(PARROT_INTERP, PMC *pmc, INTVAL value, PMC *dest)
{
#line 859 "./src/pmc/complex.pmc"

    return VTABLE_multiply_float(interp, pmc, (FLOATVAL) value, dest);
#line 982 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_neg(PARROT_INTERP, PMC *pmc, PMC *dest)
{
#line 1091 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    dest = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, dest, -re);
    SETATTR_Complex_im(interp, dest, -im);

    return dest;
#line 999 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_set_integer_keyed(PARROT_INTERP, PMC *pmc, PMC *key, INTVAL value)
{
#line 586 "./src/pmc/complex.pmc"

    VTABLE_set_number_keyed(interp, pmc, key, (FLOATVAL)value);
#line 1007 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_set_integer_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key, INTVAL value)
{
#line 590 "./src/pmc/complex.pmc"

    VTABLE_set_number_keyed_str(interp, pmc, key, (FLOATVAL)value);
#line 1015 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_set_integer_native(PARROT_INTERP, PMC *pmc, INTVAL value)
{
#line 556 "./src/pmc/complex.pmc"

    VTABLE_set_number_native(interp, pmc, (FLOATVAL)value);
#line 1023 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_set_number_keyed(PARROT_INTERP, PMC *pmc, PMC *key, FLOATVAL value)
{
#line 594 "./src/pmc/complex.pmc"

    if (VTABLE_isa(interp, key, CONST_STRING(interp, "Integer"))) {
        const INTVAL i = VTABLE_get_integer(interp, key);
        VTABLE_set_number_keyed_int(interp, pmc, i, value);
    }
    else {
        STRING *s = VTABLE_get_string(interp, key);
        VTABLE_set_number_keyed_str(interp, pmc, s, value);
    }
#line 1038 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_set_number_keyed_int(PARROT_INTERP, PMC *pmc, INTVAL key, FLOATVAL v)
{
#line 500 "./src/pmc/complex.pmc"

    switch (key) {
      case 0:
        SETATTR_Complex_re(interp, pmc, v);
        break;
      case 1:
        SETATTR_Complex_im(interp, pmc, v);
        break;
      default:
        Parrot_ex_throw_from_c_args(interp, NULL,
                EXCEPTION_INVALID_OPERATION, "Complex: key must be 0 or 1");
    }
#line 1056 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_set_number_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key, FLOATVAL value)
{
#line 605 "./src/pmc/complex.pmc"

    if (Parrot_str_equal(interp, key, CONST_STRING(interp, "real"))) {
        SETATTR_Complex_re(interp, pmc, value);
    }
    else if (Parrot_str_equal(interp, key, CONST_STRING(interp, "imag"))) {
        SETATTR_Complex_im(interp, pmc, value);
    }
    else
        Parrot_ex_throw_from_c_args(interp, NULL,
            EXCEPTION_INVALID_OPERATION, "Complex: key is neither 'real' or 'imag'");
#line 1072 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_set_number_native(PARROT_INTERP, PMC *pmc, FLOATVAL value)
{
#line 560 "./src/pmc/complex.pmc"

    SETATTR_Complex_re(interp, pmc, value);
    SETATTR_Complex_im(interp, pmc, 0.0);
#line 1081 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_set_pmc(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 544 "./src/pmc/complex.pmc"

    if (VTABLE_isa(interp, value, CONST_STRING(interp, "Complex"))) {
        FLOATVAL re, im;
        GETATTR_Complex_re(interp, value, re);
        GETATTR_Complex_im(interp, value, im);
        SETATTR_Complex_re(interp, pmc, re);
        SETATTR_Complex_im(interp, pmc, im);
    }
    else
        VTABLE_set_string_native(interp, pmc, VTABLE_get_string(interp, value));
#line 1097 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_set_pmc_keyed(PARROT_INTERP, PMC *pmc, PMC *key, PMC *value)
{
#line 617 "./src/pmc/complex.pmc"

    const FLOATVAL f = VTABLE_get_number(interp, value);
    VTABLE_set_number_keyed(interp, pmc, key, f);
#line 1106 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_set_pmc_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key, PMC *value)
{
#line 622 "./src/pmc/complex.pmc"

    const FLOATVAL f = VTABLE_get_number(interp, value);
    VTABLE_set_number_keyed_str(interp, pmc, key, f);
#line 1115 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_set_string_native(PARROT_INTERP, PMC *pmc, STRING *value)
{
#line 537 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    complex_parse_string(interp, &re, &im, value);
    SETATTR_Complex_re(interp, pmc, re);
    SETATTR_Complex_im(interp, pmc, im);
#line 1126 "./src/pmc/complex.c"
}
static  PMC *
Parrot_Complex_subtract(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
    INTVAL type = VTABLE_type(interp, value);
    /* For dynpmc fallback to MMD */
    if ((type >= enum_class_core_max) || (VTABLE_type(interp, pmc) >= enum_class_core_max))
        type = enum_class_core_max;
    switch(type) {
        case enum_class_Complex:
            return Parrot_Complex_multi_subtract_Complex_PMC(interp, pmc, value, dest);
            break;

        default:
            if (type < enum_class_core_max)
                return Parrot_Complex_multi_subtract_DEFAULT_PMC(interp, pmc, value, dest);
            else {
                PMC *retval = PMCNULL;
                Parrot_mmd_multi_dispatch_from_c_args(interp, "subtract", "PPP->P", pmc, value, dest, &retval);
                return retval;
            }
            break;

    }
}
static  PMC  *
Parrot_Complex_subtract_float(PARROT_INTERP, PMC *pmc, FLOATVAL value, PMC *dest)
{
#line 762 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    dest = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, dest, re - value);
    SETATTR_Complex_im(interp, dest, im);

    return dest;
#line 1166 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_subtract_int(PARROT_INTERP, PMC *pmc, INTVAL value, PMC *dest)
{
#line 758 "./src/pmc/complex.pmc"

    return VTABLE_subtract_float(interp, pmc, (FLOATVAL)value, dest);
#line 1174 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_thaw(PARROT_INTERP, PMC *pmc, PMC *visit)
{
#line 323 "./src/pmc/complex.pmc"

    FLOATVAL re, im;

    interp->vtables[enum_class_default]->thaw(interp, pmc, visit);

    re = VTABLE_shift_float(interp, visit);
    SETATTR_Complex_re(interp, pmc, re);

    im = VTABLE_shift_float(interp, visit);
    SETATTR_Complex_im(interp, pmc, im);
#line 1190 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_multi_add_Complex_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
#line 641 "./src/pmc/complex.pmc"

    FLOATVAL self_re, self_im, val_re, val_im;
    dest = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, self_re);
    GETATTR_Complex_im(interp, pmc, self_im);

    GETATTR_Complex_re(interp, value, val_re);
    GETATTR_Complex_im(interp, value, val_im);

    SETATTR_Complex_re(interp, dest, self_re + val_re);
    SETATTR_Complex_im(interp, dest, self_im + val_im);

    return dest;
#line 1210 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_multi_add_DEFAULT_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
#line 657 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    dest = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, dest, re + VTABLE_get_number(interp, value));
    SETATTR_Complex_im(interp, dest, im);

    return dest;
#line 1226 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_multi_i_add_Complex(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 685 "./src/pmc/complex.pmc"

    FLOATVAL self_re, self_im, val_re, val_im;

    GETATTR_Complex_re(interp, pmc, self_re);
    GETATTR_Complex_im(interp, pmc, self_im);

    GETATTR_Complex_re(interp, value, val_re);
    GETATTR_Complex_im(interp, value, val_im);

    SETATTR_Complex_re(interp, pmc, self_re + val_re);
    SETATTR_Complex_im(interp, pmc, self_im + val_im);
#line 1243 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_multi_i_add_DEFAULT(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 698 "./src/pmc/complex.pmc"

    FLOATVAL re;

    GETATTR_Complex_re(interp, pmc, re);
    SETATTR_Complex_re(interp, pmc, re + VTABLE_get_number(interp, value));
#line 1254 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_multi_subtract_Complex_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
#line 730 "./src/pmc/complex.pmc"

    FLOATVAL self_re, self_im, val_re, val_im;
    dest = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, self_re);
    GETATTR_Complex_im(interp, pmc, self_im);

    GETATTR_Complex_re(interp, value, val_re);
    GETATTR_Complex_im(interp, value, val_im);

    SETATTR_Complex_re(interp, dest, self_re - val_re);
    SETATTR_Complex_im(interp, dest, self_im - val_im);

    return dest;
#line 1274 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_multi_subtract_DEFAULT_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
#line 746 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    dest = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, dest, re - VTABLE_get_number(interp, value));
    SETATTR_Complex_im(interp, dest, im);

    return dest;
#line 1290 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_multi_i_subtract_Complex(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 774 "./src/pmc/complex.pmc"

    FLOATVAL self_re, self_im, val_re, val_im;

    GETATTR_Complex_re(interp, pmc, self_re);
    GETATTR_Complex_im(interp, pmc, self_im);

    GETATTR_Complex_re(interp, value, val_re);
    GETATTR_Complex_im(interp, value, val_im);

    SETATTR_Complex_re(interp, pmc, self_re - val_re);
    SETATTR_Complex_im(interp, pmc, self_im - val_im);
#line 1307 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_multi_i_subtract_DEFAULT(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 787 "./src/pmc/complex.pmc"

    FLOATVAL re;

    GETATTR_Complex_re(interp, pmc, re);
    SETATTR_Complex_re(interp, pmc, re - VTABLE_get_number(interp, value));
#line 1318 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_multi_multiply_Complex_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
#line 832 "./src/pmc/complex.pmc"

    FLOATVAL a, b, c, d;

    dest = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, a);
    GETATTR_Complex_im(interp, pmc, b);
    GETATTR_Complex_re(interp, value, c);
    GETATTR_Complex_im(interp, value, d);
    SETATTR_Complex_re(interp, dest, a * c - b * d);
    SETATTR_Complex_im(interp, dest, a * d + b * c);

    return dest;
#line 1337 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_multi_multiply_DEFAULT_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
#line 847 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    dest = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, dest, re * VTABLE_get_number(interp, value));
    SETATTR_Complex_im(interp, dest, im * VTABLE_get_number(interp, value));

    return dest;
#line 1353 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_multi_i_multiply_Complex(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 875 "./src/pmc/complex.pmc"

    FLOATVAL a, b, c, d;

    GETATTR_Complex_re(interp, pmc, a);
    GETATTR_Complex_im(interp, pmc, b);
    GETATTR_Complex_re(interp, value, c);
    GETATTR_Complex_im(interp, value, d);
    SETATTR_Complex_re(interp, pmc, a * c - b * d);
    SETATTR_Complex_im(interp, pmc, a * d + b * c);
#line 1368 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_multi_i_multiply_DEFAULT(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 886 "./src/pmc/complex.pmc"

    FLOATVAL re, im;

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, pmc, re * VTABLE_get_number(interp, value));
    SETATTR_Complex_im(interp, pmc, im * VTABLE_get_number(interp, value));
#line 1381 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_multi_divide_Complex_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
#line 943 "./src/pmc/complex.pmc"

    FLOATVAL mod, re, im;
    FLOATVAL self_re, self_im, val_re, val_im;

    complex_check_divide_zero(interp, value);
    dest = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, self_re);
    GETATTR_Complex_im(interp, pmc, self_im);
    GETATTR_Complex_re(interp, value, val_re);
    GETATTR_Complex_im(interp, value, val_im);

    /* a little speed optimisation: cache an intermediate number;
        I'm not sure the compiler does this */

    if (self_im == 0.0 && val_im == 0.0) {
        re = self_re / val_re;
        im = 0.0;
    }
    else {
        mod = (val_re * val_re + val_im * val_im);
        re  = (self_re * val_re + self_im * val_im) / mod;
        im  = (self_im * val_re - self_re * val_im) / mod;
    }

    SETATTR_Complex_re(interp, dest, re);
    SETATTR_Complex_im(interp, dest, im);

    return dest;
#line 1416 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_multi_divide_DEFAULT_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
#line 974 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    const FLOATVAL d = VTABLE_get_number(interp, value);
    float_check_divide_zero(interp, d);
    dest = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, dest, re / d);
    SETATTR_Complex_im(interp, dest, im / d);

    return dest;
#line 1435 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_multi_i_divide_Complex(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 1017 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    FLOATVAL self_re, self_im, val_re, val_im;

    complex_check_divide_zero(interp, value);

    GETATTR_Complex_re(interp, pmc, self_re);
    GETATTR_Complex_im(interp, pmc, self_im);
    GETATTR_Complex_re(interp, value, val_re);
    GETATTR_Complex_im(interp, value, val_im);

    if (self_im == 0.0 && val_im == 0.0) {
        re = self_re / val_re;
        im = 0.0;
    }
    else {
        /* a little speed optimisation: cache an intermediate number;
           I'm not sure the compiler does this */
        const FLOATVAL mod = (val_re * val_re + val_im * val_im);
        re  = (self_re * val_re + self_im * val_im) / mod;
        im  = (self_im * val_re - self_re * val_im) / mod;
    }

    SETATTR_Complex_re(interp, pmc, re);
    SETATTR_Complex_im(interp, pmc, im);

#line 1467 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_multi_i_divide_DEFAULT(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 1045 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    const FLOATVAL d = VTABLE_get_number(interp, value);
    float_check_divide_zero(interp, d);

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, pmc, re / d);
    SETATTR_Complex_im(interp, pmc, im / d);
#line 1483 "./src/pmc/complex.c"
}
static  INTVAL 
Parrot_Complex_multi_is_equal_Complex(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 1122 "./src/pmc/complex.pmc"

    FLOATVAL self_re, self_im, val_re, val_im;
    GETATTR_Complex_re(interp, pmc, self_re);
    GETATTR_Complex_im(interp, pmc, self_im);
    GETATTR_Complex_re(interp, value, val_re);
    GETATTR_Complex_im(interp, value, val_im);
    return (INTVAL)(self_re == val_re && self_im == val_im);
#line 1496 "./src/pmc/complex.c"
}
static  INTVAL 
Parrot_Complex_multi_is_equal_DEFAULT(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 1131 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    if (im != 0.0)
        return (INTVAL)0;

    return (re == VTABLE_get_number(interp, value));
#line 1511 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_ln(PARROT_INTERP, PMC *pmc)
{
#line 423 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    PMC * const _ctx         = CURRENT_CONTEXT(interp);
    PMC * const _ccont       = Parrot_pcc_get_continuation(interp, _ctx);
    PMC * const _call_object = Parrot_pcc_get_signature(interp, _ctx);

    Parrot_pcc_set_signature(interp, _ctx, NULL);

    { /* BEGIN PARMS SCOPE */
#line 1524 "./src/pmc/complex.pmc"
PMC* pmc;

#line 431 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
        Parrot_pcc_fill_params_from_c_args(interp, _call_object, "Pi",
            &pmc);
#line 435 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    { /* BEGIN PMETHOD BODY */
#line 1209 "./src/pmc/complex.pmc"

    PMC * const d  = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));
    FLOATVAL re, im, result_re, result_im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    /* This is necessary for atan2 to behave */
    if (im == -0.0)
        im = 0.0;

    result_re = log(sqrt(re*re + im*im));
    if (re == 0.0 && im == 0.0) /* atan2(0, 0) not portable */
        result_im = 0.0;
    else
        result_im = atan2(im, re);

    SETATTR_Complex_re(interp, d, result_re);
    SETATTR_Complex_im(interp, d, result_im);

    #line 260 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
#line 267 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    Parrot_pcc_fill_returns_from_c_args(interp, _call_object, "P",
            (PMC*)d);
    return;
    /*END RETURN PMC *d */
    }
#line 1228 "./src/pmc/complex.pmc"

#line 442 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    } /* END PMETHOD BODY */
    } /* END PARAMS SCOPE */
    return;
#line 1568 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_exp(PARROT_INTERP, PMC *pmc)
{
#line 423 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    PMC * const _ctx         = CURRENT_CONTEXT(interp);
    PMC * const _ccont       = Parrot_pcc_get_continuation(interp, _ctx);
    PMC * const _call_object = Parrot_pcc_get_signature(interp, _ctx);

    Parrot_pcc_set_signature(interp, _ctx, NULL);

    { /* BEGIN PARMS SCOPE */
#line 1581 "./src/pmc/complex.pmc"
PMC* pmc;

#line 431 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
        Parrot_pcc_fill_params_from_c_args(interp, _call_object, "Pi",
            &pmc);
#line 435 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    { /* BEGIN PMETHOD BODY */
#line 1243 "./src/pmc/complex.pmc"

    PMC * const d  = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));
    FLOATVAL re, im, f;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    f  = exp(re);

    SETATTR_Complex_re(interp, d, f * cos(im));

    /* If only sin(pi) worked. */
    if (im == 4.0 * atan(1.0)) {
        SETATTR_Complex_im(interp, d, 0.0);
    }
    else {
        SETATTR_Complex_im(interp, d, f * sin(im));
    }

    #line 260 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
#line 267 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    Parrot_pcc_fill_returns_from_c_args(interp, _call_object, "P",
            (PMC*)d);
    return;
    /*END RETURN PMC *d */
    }
#line 1261 "./src/pmc/complex.pmc"

#line 442 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    } /* END PMETHOD BODY */
    } /* END PARAMS SCOPE */
    return;
#line 1624 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_sin(PARROT_INTERP, PMC *pmc)
{
#line 423 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    PMC * const _ctx         = CURRENT_CONTEXT(interp);
    PMC * const _ccont       = Parrot_pcc_get_continuation(interp, _ctx);
    PMC * const _call_object = Parrot_pcc_get_signature(interp, _ctx);

    Parrot_pcc_set_signature(interp, _ctx, NULL);

    { /* BEGIN PARMS SCOPE */
#line 1637 "./src/pmc/complex.pmc"
PMC* pmc;

#line 431 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
        Parrot_pcc_fill_params_from_c_args(interp, _call_object, "Pi",
            &pmc);
#line 435 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    { /* BEGIN PMETHOD BODY */
#line 1297 "./src/pmc/complex.pmc"

    PMC * const d = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));
    FLOATVAL re, im, result_re, result_im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    if (FLOAT_IS_ZERO(im)) {
        result_re = sin(re);
        result_im = 0.0;
    }
    else if (FLOAT_IS_ZERO(re)) {
        result_re = 0.0;
        result_im = sinh(im);
    }
    else {
        result_re = sin(re) * cosh(im);

        if (im == -0.0)
            result_im = 0.0;
        else
            result_im = cos(re) * sinh(im);

    }

    SETATTR_Complex_re(interp, d, result_re);
    SETATTR_Complex_im(interp, d, result_im);

    #line 260 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
#line 267 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    Parrot_pcc_fill_returns_from_c_args(interp, _call_object, "P",
            (PMC*)d);
    return;
    /*END RETURN PMC *d */
    }
#line 1324 "./src/pmc/complex.pmc"

#line 442 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    } /* END PMETHOD BODY */
    } /* END PARAMS SCOPE */
    return;
#line 1689 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_cos(PARROT_INTERP, PMC *pmc)
{
#line 423 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    PMC * const _ctx         = CURRENT_CONTEXT(interp);
    PMC * const _ccont       = Parrot_pcc_get_continuation(interp, _ctx);
    PMC * const _call_object = Parrot_pcc_get_signature(interp, _ctx);

    Parrot_pcc_set_signature(interp, _ctx, NULL);

    { /* BEGIN PARMS SCOPE */
#line 1702 "./src/pmc/complex.pmc"
PMC* pmc;

#line 431 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
        Parrot_pcc_fill_params_from_c_args(interp, _call_object, "Pi",
            &pmc);
#line 435 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    { /* BEGIN PMETHOD BODY */
#line 1327 "./src/pmc/complex.pmc"

    PMC * const d  = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));
    FLOATVAL re, im, result_re, result_im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    if (FLOAT_IS_ZERO(re)) {
        result_re = cosh(im);
        result_im = 0.0;
    }
    else if (FLOAT_IS_ZERO(im)) {
        result_re = cos(re);
        result_im = 0.0;
    }
    else {
        result_re = cos(re) * cosh(im);
        result_im = -1.0 * sin(re) * sinh(im);
    }

    SETATTR_Complex_re(interp, d, result_re);
    SETATTR_Complex_im(interp, d, result_im);

    #line 260 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
#line 267 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    Parrot_pcc_fill_returns_from_c_args(interp, _call_object, "P",
            (PMC*)d);
    return;
    /*END RETURN PMC *d */
    }
#line 1349 "./src/pmc/complex.pmc"

#line 442 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    } /* END PMETHOD BODY */
    } /* END PARAMS SCOPE */
    return;
#line 1749 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_tan(PARROT_INTERP, PMC *pmc)
{
#line 423 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    PMC * const _ctx         = CURRENT_CONTEXT(interp);
    PMC * const _ccont       = Parrot_pcc_get_continuation(interp, _ctx);
    PMC * const _call_object = Parrot_pcc_get_signature(interp, _ctx);

    Parrot_pcc_set_signature(interp, _ctx, NULL);

    { /* BEGIN PARMS SCOPE */
#line 1762 "./src/pmc/complex.pmc"
PMC* pmc;

#line 431 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
        Parrot_pcc_fill_params_from_c_args(interp, _call_object, "Pi",
            &pmc);
#line 435 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    { /* BEGIN PMETHOD BODY */
#line 1352 "./src/pmc/complex.pmc"

    PMC * const d = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));
    PMC * const e = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));

    #line 1775 "./src/pmc/complex.pmc"
Parrot_pcc_invoke_method_from_c_args(interp, pmc, CONST_STRING_GEN(interp, "sin"), "->P", &d);
#line 1356 "./src/pmc/complex.pmc"

    #line 1779 "./src/pmc/complex.pmc"
Parrot_pcc_invoke_method_from_c_args(interp, pmc, CONST_STRING_GEN(interp, "cos"), "->P", &e);
#line 1357 "./src/pmc/complex.pmc"


    Parrot_Complex_multi_i_divide_Complex(interp, d, e);

    #line 260 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
#line 267 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    Parrot_pcc_fill_returns_from_c_args(interp, _call_object, "P",
            (PMC*)d);
    return;
    /*END RETURN PMC *d */
    }
#line 1361 "./src/pmc/complex.pmc"

#line 442 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    } /* END PMETHOD BODY */
    } /* END PARAMS SCOPE */
    return;
#line 1802 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_cot(PARROT_INTERP, PMC *pmc)
{
#line 423 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    PMC * const _ctx         = CURRENT_CONTEXT(interp);
    PMC * const _ccont       = Parrot_pcc_get_continuation(interp, _ctx);
    PMC * const _call_object = Parrot_pcc_get_signature(interp, _ctx);

    Parrot_pcc_set_signature(interp, _ctx, NULL);

    { /* BEGIN PARMS SCOPE */
#line 1815 "./src/pmc/complex.pmc"
PMC* pmc;

#line 431 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
        Parrot_pcc_fill_params_from_c_args(interp, _call_object, "Pi",
            &pmc);
#line 435 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    { /* BEGIN PMETHOD BODY */
#line 1364 "./src/pmc/complex.pmc"

    PMC * const d = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));
    PMC * const e = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));
    #line 1827 "./src/pmc/complex.pmc"
Parrot_pcc_invoke_method_from_c_args(interp, pmc, CONST_STRING_GEN(interp, "cos"), "->P", &d);
#line 1367 "./src/pmc/complex.pmc"

    #line 1831 "./src/pmc/complex.pmc"
Parrot_pcc_invoke_method_from_c_args(interp, pmc, CONST_STRING_GEN(interp, "sin"), "->P", &e);
#line 1368 "./src/pmc/complex.pmc"


    Parrot_Complex_multi_i_divide_Complex(interp, d, e);

    #line 260 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
#line 267 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    Parrot_pcc_fill_returns_from_c_args(interp, _call_object, "P",
            (PMC*)d);
    return;
    /*END RETURN PMC *d */
    }
#line 1372 "./src/pmc/complex.pmc"

#line 442 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    } /* END PMETHOD BODY */
    } /* END PARAMS SCOPE */
    return;
#line 1854 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_sec(PARROT_INTERP, PMC *pmc)
{
#line 423 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    PMC * const _ctx         = CURRENT_CONTEXT(interp);
    PMC * const _ccont       = Parrot_pcc_get_continuation(interp, _ctx);
    PMC * const _call_object = Parrot_pcc_get_signature(interp, _ctx);

    Parrot_pcc_set_signature(interp, _ctx, NULL);

    { /* BEGIN PARMS SCOPE */
#line 1867 "./src/pmc/complex.pmc"
PMC* pmc;

#line 431 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
        Parrot_pcc_fill_params_from_c_args(interp, _call_object, "Pi",
            &pmc);
#line 435 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    { /* BEGIN PMETHOD BODY */
#line 1375 "./src/pmc/complex.pmc"

    PMC * const d = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));
    PMC * const e = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));
    SETATTR_Complex_re(interp, d, 1.0);
    SETATTR_Complex_im(interp, d, 0.0);
    #line 1881 "./src/pmc/complex.pmc"
Parrot_pcc_invoke_method_from_c_args(interp, pmc, CONST_STRING_GEN(interp, "cos"), "->P", &e);
#line 1380 "./src/pmc/complex.pmc"


    Parrot_Complex_multi_i_divide_Complex(interp, d, e);

    #line 260 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
#line 267 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    Parrot_pcc_fill_returns_from_c_args(interp, _call_object, "P",
            (PMC*)d);
    return;
    /*END RETURN PMC *d */
    }
#line 1384 "./src/pmc/complex.pmc"

#line 442 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    } /* END PMETHOD BODY */
    } /* END PARAMS SCOPE */
    return;
#line 1904 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_csc(PARROT_INTERP, PMC *pmc)
{
#line 423 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    PMC * const _ctx         = CURRENT_CONTEXT(interp);
    PMC * const _ccont       = Parrot_pcc_get_continuation(interp, _ctx);
    PMC * const _call_object = Parrot_pcc_get_signature(interp, _ctx);

    Parrot_pcc_set_signature(interp, _ctx, NULL);

    { /* BEGIN PARMS SCOPE */
#line 1917 "./src/pmc/complex.pmc"
PMC* pmc;

#line 431 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
        Parrot_pcc_fill_params_from_c_args(interp, _call_object, "Pi",
            &pmc);
#line 435 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    { /* BEGIN PMETHOD BODY */
#line 1387 "./src/pmc/complex.pmc"

    PMC * const d = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));
    PMC * const e = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));

    SETATTR_Complex_re(interp, d, 1.0);
    SETATTR_Complex_im(interp, d, 0.0);

    #line 1933 "./src/pmc/complex.pmc"
Parrot_pcc_invoke_method_from_c_args(interp, pmc, CONST_STRING_GEN(interp, "sin"), "->P", &e);
#line 1394 "./src/pmc/complex.pmc"


    Parrot_Complex_multi_i_divide_Complex(interp, d, e);

    #line 260 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
#line 267 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    Parrot_pcc_fill_returns_from_c_args(interp, _call_object, "P",
            (PMC*)d);
    return;
    /*END RETURN PMC *d */
    }
#line 1398 "./src/pmc/complex.pmc"

#line 442 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    } /* END PMETHOD BODY */
    } /* END PARAMS SCOPE */
    return;
#line 1956 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_asin(PARROT_INTERP, PMC *pmc)
{
#line 423 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    PMC * const _ctx         = CURRENT_CONTEXT(interp);
    PMC * const _ccont       = Parrot_pcc_get_continuation(interp, _ctx);
    PMC * const _call_object = Parrot_pcc_get_signature(interp, _ctx);

    Parrot_pcc_set_signature(interp, _ctx, NULL);

    { /* BEGIN PARMS SCOPE */
#line 1969 "./src/pmc/complex.pmc"
PMC* pmc;

#line 431 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
        Parrot_pcc_fill_params_from_c_args(interp, _call_object, "Pi",
            &pmc);
#line 435 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    { /* BEGIN PMETHOD BODY */
#line 1434 "./src/pmc/complex.pmc"

    FLOATVAL d_re, d_im, e_re, e_im, self_re, self_im;
    PMC * const d = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));
    PMC *       e = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));
    GETATTR_Complex_re(interp, pmc, self_re);
    GETATTR_Complex_im(interp, pmc, self_im);

    e      = Parrot_Complex_multi_multiply_Complex_PMC(interp, pmc, pmc, e);
    GETATTR_Complex_re(interp, e, e_re);
    GETATTR_Complex_im(interp, e, e_im);
    SETATTR_Complex_re(interp, e, 1.0 - e_re);
    SETATTR_Complex_im(interp, e, -e_im);

    #line 1991 "./src/pmc/complex.pmc"
Parrot_pcc_invoke_method_from_c_args(interp, e, CONST_STRING_GEN(interp, "sqrt"), "->P", &d);
#line 1447 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, d, d_re);
    GETATTR_Complex_im(interp, d, d_im);
    SETATTR_Complex_re(interp, d, d_re - self_im);
    SETATTR_Complex_im(interp, d, d_im + self_re);

    #line 2000 "./src/pmc/complex.pmc"
Parrot_pcc_invoke_method_from_c_args(interp, d, CONST_STRING_GEN(interp, "ln"), "->P", &d);
#line 1453 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, d, d_re);
    GETATTR_Complex_im(interp, d, d_im);
    SETATTR_Complex_re(interp, e, d_im);
    SETATTR_Complex_im(interp, e, d_re ? -d_re : 0.0);

    #line 260 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
#line 267 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    Parrot_pcc_fill_returns_from_c_args(interp, _call_object, "P",
            (PMC*)e);
    return;
    /*END RETURN PMC *e */
    }
#line 1459 "./src/pmc/complex.pmc"

#line 442 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    } /* END PMETHOD BODY */
    } /* END PARAMS SCOPE */
    return;
#line 2025 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_acos(PARROT_INTERP, PMC *pmc)
{
#line 423 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    PMC * const _ctx         = CURRENT_CONTEXT(interp);
    PMC * const _ccont       = Parrot_pcc_get_continuation(interp, _ctx);
    PMC * const _call_object = Parrot_pcc_get_signature(interp, _ctx);

    Parrot_pcc_set_signature(interp, _ctx, NULL);

    { /* BEGIN PARMS SCOPE */
#line 2038 "./src/pmc/complex.pmc"
PMC* pmc;

#line 431 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
        Parrot_pcc_fill_params_from_c_args(interp, _call_object, "Pi",
            &pmc);
#line 435 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    { /* BEGIN PMETHOD BODY */
#line 1462 "./src/pmc/complex.pmc"

    FLOATVAL d_re, d_im, e_re, e_im, self_re, self_im;
    PMC * const d = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));
    PMC *       e = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));
    GETATTR_Complex_re(interp, pmc, self_re);
    GETATTR_Complex_im(interp, pmc, self_im);

    e = Parrot_Complex_multi_multiply_Complex_PMC(interp, pmc, pmc, e);
    GETATTR_Complex_re(interp, e, e_re);
    GETATTR_Complex_im(interp, e, e_im);
    SETATTR_Complex_re(interp, e, 1.0 - e_re);
    SETATTR_Complex_im(interp, e, -e_im);

    #line 2060 "./src/pmc/complex.pmc"
Parrot_pcc_invoke_method_from_c_args(interp, e, CONST_STRING_GEN(interp, "sqrt"), "->P", &d);
#line 1475 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, d, d_re);
    GETATTR_Complex_im(interp, d, d_im);
    SETATTR_Complex_re(interp, d, d_re + self_im);
    SETATTR_Complex_im(interp, d, d_im - self_re);

    #line 2069 "./src/pmc/complex.pmc"
Parrot_pcc_invoke_method_from_c_args(interp, d, CONST_STRING_GEN(interp, "ln"), "->P", &e);
#line 1481 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, e, e_re);
    GETATTR_Complex_im(interp, e, e_im);
    SETATTR_Complex_re(interp, d, e_im + 2.0 * atan(1.0));
    SETATTR_Complex_im(interp, d, e_re ? -e_re : 0.0);

    #line 260 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
#line 267 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    Parrot_pcc_fill_returns_from_c_args(interp, _call_object, "P",
            (PMC*)d);
    return;
    /*END RETURN PMC *d */
    }
#line 1487 "./src/pmc/complex.pmc"

#line 442 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    } /* END PMETHOD BODY */
    } /* END PARAMS SCOPE */
    return;
#line 2094 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_atan(PARROT_INTERP, PMC *pmc)
{
#line 423 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    PMC * const _ctx         = CURRENT_CONTEXT(interp);
    PMC * const _ccont       = Parrot_pcc_get_continuation(interp, _ctx);
    PMC * const _call_object = Parrot_pcc_get_signature(interp, _ctx);

    Parrot_pcc_set_signature(interp, _ctx, NULL);

    { /* BEGIN PARMS SCOPE */
#line 2107 "./src/pmc/complex.pmc"
PMC* pmc;

#line 431 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
        Parrot_pcc_fill_params_from_c_args(interp, _call_object, "Pi",
            &pmc);
#line 435 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    { /* BEGIN PMETHOD BODY */
#line 1490 "./src/pmc/complex.pmc"

    PMC * const d  = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));
    PMC * const e  = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));
    FLOATVAL re, im, d_re, d_im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, d, re);
    SETATTR_Complex_im(interp, d, 1 + im);
    SETATTR_Complex_re(interp, e, -re);
    SETATTR_Complex_im(interp, e, 1 - im);

    Parrot_Complex_multi_i_divide_Complex(interp, d, e);

    #line 2130 "./src/pmc/complex.pmc"
Parrot_pcc_invoke_method_from_c_args(interp, d, CONST_STRING_GEN(interp, "ln"), "->P", &d);
#line 1504 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, d, d_re);
    GETATTR_Complex_im(interp, d, d_im);

    SETATTR_Complex_re(interp, e, (d_im ? d_im : -0.0) / -2.0);
    SETATTR_Complex_im(interp, e, d_re / 2.0);

    #line 260 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
#line 267 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    Parrot_pcc_fill_returns_from_c_args(interp, _call_object, "P",
            (PMC*)e);
    return;
    /*END RETURN PMC *e */
    }
#line 1511 "./src/pmc/complex.pmc"

#line 442 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    } /* END PMETHOD BODY */
    } /* END PARAMS SCOPE */
    return;
#line 2156 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_acot(PARROT_INTERP, PMC *pmc)
{
#line 423 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    PMC * const _ctx         = CURRENT_CONTEXT(interp);
    PMC * const _ccont       = Parrot_pcc_get_continuation(interp, _ctx);
    PMC * const _call_object = Parrot_pcc_get_signature(interp, _ctx);

    Parrot_pcc_set_signature(interp, _ctx, NULL);

    { /* BEGIN PARMS SCOPE */
#line 2169 "./src/pmc/complex.pmc"
PMC* pmc;

#line 431 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
        Parrot_pcc_fill_params_from_c_args(interp, _call_object, "Pi",
            &pmc);
#line 435 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    { /* BEGIN PMETHOD BODY */
#line 1514 "./src/pmc/complex.pmc"

    PMC * const d = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));
    PMC     *e;
    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, d,  re / (re * re + im * im));
    SETATTR_Complex_im(interp, d, -im / (re * re + im * im));

    #line 2188 "./src/pmc/complex.pmc"
Parrot_pcc_invoke_method_from_c_args(interp, d, CONST_STRING_GEN(interp, "atan"), "->P", &e);
#line 1524 "./src/pmc/complex.pmc"

    #line 260 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
#line 267 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    Parrot_pcc_fill_returns_from_c_args(interp, _call_object, "P",
            (PMC*)e);
    return;
    /*END RETURN PMC *e */
    }
#line 1525 "./src/pmc/complex.pmc"

#line 442 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    } /* END PMETHOD BODY */
    } /* END PARAMS SCOPE */
    return;
#line 2208 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_acsc(PARROT_INTERP, PMC *pmc)
{
#line 423 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    PMC * const _ctx         = CURRENT_CONTEXT(interp);
    PMC * const _ccont       = Parrot_pcc_get_continuation(interp, _ctx);
    PMC * const _call_object = Parrot_pcc_get_signature(interp, _ctx);

    Parrot_pcc_set_signature(interp, _ctx, NULL);

    { /* BEGIN PARMS SCOPE */
#line 2221 "./src/pmc/complex.pmc"
PMC* pmc;

#line 431 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
        Parrot_pcc_fill_params_from_c_args(interp, _call_object, "Pi",
            &pmc);
#line 435 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    { /* BEGIN PMETHOD BODY */
#line 1528 "./src/pmc/complex.pmc"

    PMC * const d = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));
    PMC     *e;
    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, d,  re / (re * re + im * im));
    SETATTR_Complex_im(interp, d, -im / (re * re + im * im));

    #line 2240 "./src/pmc/complex.pmc"
Parrot_pcc_invoke_method_from_c_args(interp, d, CONST_STRING_GEN(interp, "asin"), "->P", &e);
#line 1538 "./src/pmc/complex.pmc"

    #line 260 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
#line 267 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    Parrot_pcc_fill_returns_from_c_args(interp, _call_object, "P",
            (PMC*)e);
    return;
    /*END RETURN PMC *e */
    }
#line 1539 "./src/pmc/complex.pmc"

#line 442 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    } /* END PMETHOD BODY */
    } /* END PARAMS SCOPE */
    return;
#line 2260 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_asec(PARROT_INTERP, PMC *pmc)
{
#line 423 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    PMC * const _ctx         = CURRENT_CONTEXT(interp);
    PMC * const _ccont       = Parrot_pcc_get_continuation(interp, _ctx);
    PMC * const _call_object = Parrot_pcc_get_signature(interp, _ctx);

    Parrot_pcc_set_signature(interp, _ctx, NULL);

    { /* BEGIN PARMS SCOPE */
#line 2273 "./src/pmc/complex.pmc"
PMC* pmc;

#line 431 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
        Parrot_pcc_fill_params_from_c_args(interp, _call_object, "Pi",
            &pmc);
#line 435 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    { /* BEGIN PMETHOD BODY */
#line 1542 "./src/pmc/complex.pmc"

    PMC * const d = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));
    PMC     *e;
    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, d,  re / (re * re + im * im));
    SETATTR_Complex_im(interp, d, -im / (re * re + im * im));

    #line 2292 "./src/pmc/complex.pmc"
Parrot_pcc_invoke_method_from_c_args(interp, d, CONST_STRING_GEN(interp, "acos"), "->P", &e);
#line 1552 "./src/pmc/complex.pmc"

    #line 260 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
#line 267 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    Parrot_pcc_fill_returns_from_c_args(interp, _call_object, "P",
            (PMC*)e);
    return;
    /*END RETURN PMC *e */
    }
#line 1553 "./src/pmc/complex.pmc"

#line 442 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    } /* END PMETHOD BODY */
    } /* END PARAMS SCOPE */
    return;
#line 2312 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_sinh(PARROT_INTERP, PMC *pmc)
{
#line 423 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    PMC * const _ctx         = CURRENT_CONTEXT(interp);
    PMC * const _ccont       = Parrot_pcc_get_continuation(interp, _ctx);
    PMC * const _call_object = Parrot_pcc_get_signature(interp, _ctx);

    Parrot_pcc_set_signature(interp, _ctx, NULL);

    { /* BEGIN PARMS SCOPE */
#line 2325 "./src/pmc/complex.pmc"
PMC* pmc;

#line 431 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
        Parrot_pcc_fill_params_from_c_args(interp, _call_object, "Pi",
            &pmc);
#line 435 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    { /* BEGIN PMETHOD BODY */
#line 1576 "./src/pmc/complex.pmc"

    PMC * const d = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));
    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, d, sinh(re) * cos(im));
    SETATTR_Complex_im(interp, d, im ? cosh(re) * sin(im) : 0.0);

    #line 260 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
#line 267 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    Parrot_pcc_fill_returns_from_c_args(interp, _call_object, "P",
            (PMC*)d);
    return;
    /*END RETURN PMC *d */
    }
#line 1585 "./src/pmc/complex.pmc"

#line 442 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    } /* END PMETHOD BODY */
    } /* END PARAMS SCOPE */
    return;
#line 2359 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_cosh(PARROT_INTERP, PMC *pmc)
{
#line 423 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    PMC * const _ctx         = CURRENT_CONTEXT(interp);
    PMC * const _ccont       = Parrot_pcc_get_continuation(interp, _ctx);
    PMC * const _call_object = Parrot_pcc_get_signature(interp, _ctx);

    Parrot_pcc_set_signature(interp, _ctx, NULL);

    { /* BEGIN PARMS SCOPE */
#line 2372 "./src/pmc/complex.pmc"
PMC* pmc;

#line 431 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
        Parrot_pcc_fill_params_from_c_args(interp, _call_object, "Pi",
            &pmc);
#line 435 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    { /* BEGIN PMETHOD BODY */
#line 1588 "./src/pmc/complex.pmc"

    PMC * const d = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));
    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, d, cosh(re) * cos(im));
    if (re == 0.0 || im == 0.0) {
        SETATTR_Complex_im(interp, d, 0.0);
    }
    else {
        SETATTR_Complex_im(interp, d, sinh(re) * sin(im));
    }

    #line 260 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
#line 267 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    Parrot_pcc_fill_returns_from_c_args(interp, _call_object, "P",
            (PMC*)d);
    return;
    /*END RETURN PMC *d */
    }
#line 1602 "./src/pmc/complex.pmc"

#line 442 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    } /* END PMETHOD BODY */
    } /* END PARAMS SCOPE */
    return;
#line 2411 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_tanh(PARROT_INTERP, PMC *pmc)
{
#line 423 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    PMC * const _ctx         = CURRENT_CONTEXT(interp);
    PMC * const _ccont       = Parrot_pcc_get_continuation(interp, _ctx);
    PMC * const _call_object = Parrot_pcc_get_signature(interp, _ctx);

    Parrot_pcc_set_signature(interp, _ctx, NULL);

    { /* BEGIN PARMS SCOPE */
#line 2424 "./src/pmc/complex.pmc"
PMC* pmc;

#line 431 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
        Parrot_pcc_fill_params_from_c_args(interp, _call_object, "Pi",
            &pmc);
#line 435 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    { /* BEGIN PMETHOD BODY */
#line 1605 "./src/pmc/complex.pmc"

    PMC * const d = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));
    PMC * const e = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));

    #line 2437 "./src/pmc/complex.pmc"
Parrot_pcc_invoke_method_from_c_args(interp, pmc, CONST_STRING_GEN(interp, "sinh"), "->P", &d);
#line 1609 "./src/pmc/complex.pmc"

    #line 2441 "./src/pmc/complex.pmc"
Parrot_pcc_invoke_method_from_c_args(interp, pmc, CONST_STRING_GEN(interp, "cosh"), "->P", &e);
#line 1610 "./src/pmc/complex.pmc"


    Parrot_Complex_multi_i_divide_Complex(interp, d, e);

    #line 260 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
#line 267 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    Parrot_pcc_fill_returns_from_c_args(interp, _call_object, "P",
            (PMC*)d);
    return;
    /*END RETURN PMC *d */
    }
#line 1614 "./src/pmc/complex.pmc"

#line 442 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    } /* END PMETHOD BODY */
    } /* END PARAMS SCOPE */
    return;
#line 2464 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_coth(PARROT_INTERP, PMC *pmc)
{
#line 423 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    PMC * const _ctx         = CURRENT_CONTEXT(interp);
    PMC * const _ccont       = Parrot_pcc_get_continuation(interp, _ctx);
    PMC * const _call_object = Parrot_pcc_get_signature(interp, _ctx);

    Parrot_pcc_set_signature(interp, _ctx, NULL);

    { /* BEGIN PARMS SCOPE */
#line 2477 "./src/pmc/complex.pmc"
PMC* pmc;

#line 431 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
        Parrot_pcc_fill_params_from_c_args(interp, _call_object, "Pi",
            &pmc);
#line 435 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    { /* BEGIN PMETHOD BODY */
#line 1617 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    PMC * const d = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));

    #line 2490 "./src/pmc/complex.pmc"
Parrot_pcc_invoke_method_from_c_args(interp, pmc, CONST_STRING_GEN(interp, "tanh"), "->P", &d);
#line 1621 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, d, re);
    GETATTR_Complex_im(interp, d, im);

    SETATTR_Complex_re(interp, d, re ?  re / (re * re + im * im) : 0.0);
    SETATTR_Complex_im(interp, d, im ? -im / (re * re + im * im) : 0.0);

    #line 260 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
#line 267 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    Parrot_pcc_fill_returns_from_c_args(interp, _call_object, "P",
            (PMC*)d);
    return;
    /*END RETURN PMC *d */
    }
#line 1628 "./src/pmc/complex.pmc"

#line 442 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    } /* END PMETHOD BODY */
    } /* END PARAMS SCOPE */
    return;
#line 2516 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_csch(PARROT_INTERP, PMC *pmc)
{
#line 423 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    PMC * const _ctx         = CURRENT_CONTEXT(interp);
    PMC * const _ccont       = Parrot_pcc_get_continuation(interp, _ctx);
    PMC * const _call_object = Parrot_pcc_get_signature(interp, _ctx);

    Parrot_pcc_set_signature(interp, _ctx, NULL);

    { /* BEGIN PARMS SCOPE */
#line 2529 "./src/pmc/complex.pmc"
PMC* pmc;

#line 431 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
        Parrot_pcc_fill_params_from_c_args(interp, _call_object, "Pi",
            &pmc);
#line 435 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    { /* BEGIN PMETHOD BODY */
#line 1631 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    PMC * const d = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));

    #line 2542 "./src/pmc/complex.pmc"
Parrot_pcc_invoke_method_from_c_args(interp, pmc, CONST_STRING_GEN(interp, "sinh"), "->P", &d);
#line 1635 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, d, re);
    GETATTR_Complex_im(interp, d, im);

    SETATTR_Complex_re(interp, d, re ?  re / (re * re + im * im) : 0.0);
    SETATTR_Complex_im(interp, d, im ? -im / (re * re + im * im) : 0.0);

    #line 260 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
#line 267 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    Parrot_pcc_fill_returns_from_c_args(interp, _call_object, "P",
            (PMC*)d);
    return;
    /*END RETURN PMC *d */
    }
#line 1642 "./src/pmc/complex.pmc"

#line 442 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    } /* END PMETHOD BODY */
    } /* END PARAMS SCOPE */
    return;
#line 2568 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_sech(PARROT_INTERP, PMC *pmc)
{
#line 423 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    PMC * const _ctx         = CURRENT_CONTEXT(interp);
    PMC * const _ccont       = Parrot_pcc_get_continuation(interp, _ctx);
    PMC * const _call_object = Parrot_pcc_get_signature(interp, _ctx);

    Parrot_pcc_set_signature(interp, _ctx, NULL);

    { /* BEGIN PARMS SCOPE */
#line 2581 "./src/pmc/complex.pmc"
PMC* pmc;

#line 431 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
        Parrot_pcc_fill_params_from_c_args(interp, _call_object, "Pi",
            &pmc);
#line 435 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    { /* BEGIN PMETHOD BODY */
#line 1645 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    PMC * const d = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));

    #line 2594 "./src/pmc/complex.pmc"
Parrot_pcc_invoke_method_from_c_args(interp, pmc, CONST_STRING_GEN(interp, "cosh"), "->P", &d);
#line 1649 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, d, re);
    GETATTR_Complex_im(interp, d, im);

    SETATTR_Complex_re(interp, d, re ?  re / (re * re + im * im) : 0.0);
    SETATTR_Complex_im(interp, d, im ? -im / (re * re + im * im) : 0.0);

    #line 260 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
#line 267 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    Parrot_pcc_fill_returns_from_c_args(interp, _call_object, "P",
            (PMC*)d);
    return;
    /*END RETURN PMC *d */
    }
#line 1656 "./src/pmc/complex.pmc"

#line 442 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    } /* END PMETHOD BODY */
    } /* END PARAMS SCOPE */
    return;
#line 2620 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_asinh(PARROT_INTERP, PMC *pmc)
{
#line 423 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    PMC * const _ctx         = CURRENT_CONTEXT(interp);
    PMC * const _ccont       = Parrot_pcc_get_continuation(interp, _ctx);
    PMC * const _call_object = Parrot_pcc_get_signature(interp, _ctx);

    Parrot_pcc_set_signature(interp, _ctx, NULL);

    { /* BEGIN PARMS SCOPE */
#line 2633 "./src/pmc/complex.pmc"
PMC* pmc;

#line 431 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
        Parrot_pcc_fill_params_from_c_args(interp, _call_object, "Pi",
            &pmc);
#line 435 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    { /* BEGIN PMETHOD BODY */
#line 1687 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    PMC * const d = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));
    PMC * const e = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, d,  im);
    SETATTR_Complex_im(interp, d, -re);

    #line 2652 "./src/pmc/complex.pmc"
Parrot_pcc_invoke_method_from_c_args(interp, d, CONST_STRING_GEN(interp, "asin"), "->P", &d);
#line 1697 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, d, re);
    GETATTR_Complex_im(interp, d, im);
    SETATTR_Complex_re(interp, e, -im);
    SETATTR_Complex_im(interp, e,  re);

    #line 260 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
#line 267 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    Parrot_pcc_fill_returns_from_c_args(interp, _call_object, "P",
            (PMC*)e);
    return;
    /*END RETURN PMC *e */
    }
#line 1703 "./src/pmc/complex.pmc"

#line 442 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    } /* END PMETHOD BODY */
    } /* END PARAMS SCOPE */
    return;
#line 2677 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_acosh(PARROT_INTERP, PMC *pmc)
{
#line 423 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    PMC * const _ctx         = CURRENT_CONTEXT(interp);
    PMC * const _ccont       = Parrot_pcc_get_continuation(interp, _ctx);
    PMC * const _call_object = Parrot_pcc_get_signature(interp, _ctx);

    Parrot_pcc_set_signature(interp, _ctx, NULL);

    { /* BEGIN PARMS SCOPE */
#line 2690 "./src/pmc/complex.pmc"
PMC* pmc;

#line 431 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
        Parrot_pcc_fill_params_from_c_args(interp, _call_object, "Pi",
            &pmc);
#line 435 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    { /* BEGIN PMETHOD BODY */
#line 1706 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    PMC * const d = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));
    PMC * const e = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));

    #line 2704 "./src/pmc/complex.pmc"
Parrot_pcc_invoke_method_from_c_args(interp, pmc, CONST_STRING_GEN(interp, "acos"), "->P", &d);
#line 1711 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, d, re);
    GETATTR_Complex_im(interp, d, im);
    SETATTR_Complex_re(interp, e, -im);
    SETATTR_Complex_im(interp, e,  re);

    #line 260 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
#line 267 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    Parrot_pcc_fill_returns_from_c_args(interp, _call_object, "P",
            (PMC*)e);
    return;
    /*END RETURN PMC *e */
    }
#line 1717 "./src/pmc/complex.pmc"

#line 442 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    } /* END PMETHOD BODY */
    } /* END PARAMS SCOPE */
    return;
#line 2729 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_atanh(PARROT_INTERP, PMC *pmc)
{
#line 423 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    PMC * const _ctx         = CURRENT_CONTEXT(interp);
    PMC * const _ccont       = Parrot_pcc_get_continuation(interp, _ctx);
    PMC * const _call_object = Parrot_pcc_get_signature(interp, _ctx);

    Parrot_pcc_set_signature(interp, _ctx, NULL);

    { /* BEGIN PARMS SCOPE */
#line 2742 "./src/pmc/complex.pmc"
PMC* pmc;

#line 431 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
        Parrot_pcc_fill_params_from_c_args(interp, _call_object, "Pi",
            &pmc);
#line 435 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    { /* BEGIN PMETHOD BODY */
#line 1720 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    PMC * const d = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));
    PMC * const e = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, d,  im);
    SETATTR_Complex_im(interp, d, -re);

    #line 2761 "./src/pmc/complex.pmc"
Parrot_pcc_invoke_method_from_c_args(interp, d, CONST_STRING_GEN(interp, "atan"), "->P", &d);
#line 1730 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, d, re);
    GETATTR_Complex_im(interp, d, im);
    SETATTR_Complex_re(interp, e, -im);
    SETATTR_Complex_im(interp, e,  re);

    #line 260 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
#line 267 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    Parrot_pcc_fill_returns_from_c_args(interp, _call_object, "P",
            (PMC*)e);
    return;
    /*END RETURN PMC *e */
    }
#line 1736 "./src/pmc/complex.pmc"

#line 442 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    } /* END PMETHOD BODY */
    } /* END PARAMS SCOPE */
    return;
#line 2786 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_acoth(PARROT_INTERP, PMC *pmc)
{
#line 423 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    PMC * const _ctx         = CURRENT_CONTEXT(interp);
    PMC * const _ccont       = Parrot_pcc_get_continuation(interp, _ctx);
    PMC * const _call_object = Parrot_pcc_get_signature(interp, _ctx);

    Parrot_pcc_set_signature(interp, _ctx, NULL);

    { /* BEGIN PARMS SCOPE */
#line 2799 "./src/pmc/complex.pmc"
PMC* pmc;

#line 431 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
        Parrot_pcc_fill_params_from_c_args(interp, _call_object, "Pi",
            &pmc);
#line 435 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    { /* BEGIN PMETHOD BODY */
#line 1739 "./src/pmc/complex.pmc"

    PMC * const d = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));
    PMC     *e;
    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, d,  re / (re * re + im * im));
    SETATTR_Complex_im(interp, d, -im / (re * re + im * im));

    #line 2818 "./src/pmc/complex.pmc"
Parrot_pcc_invoke_method_from_c_args(interp, d, CONST_STRING_GEN(interp, "atanh"), "->P", &e);
#line 1749 "./src/pmc/complex.pmc"

    #line 260 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
#line 267 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    Parrot_pcc_fill_returns_from_c_args(interp, _call_object, "P",
            (PMC*)e);
    return;
    /*END RETURN PMC *e */
    }
#line 1750 "./src/pmc/complex.pmc"

#line 442 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    } /* END PMETHOD BODY */
    } /* END PARAMS SCOPE */
    return;
#line 2838 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_acsch(PARROT_INTERP, PMC *pmc)
{
#line 423 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    PMC * const _ctx         = CURRENT_CONTEXT(interp);
    PMC * const _ccont       = Parrot_pcc_get_continuation(interp, _ctx);
    PMC * const _call_object = Parrot_pcc_get_signature(interp, _ctx);

    Parrot_pcc_set_signature(interp, _ctx, NULL);

    { /* BEGIN PARMS SCOPE */
#line 2851 "./src/pmc/complex.pmc"
PMC* pmc;

#line 431 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
        Parrot_pcc_fill_params_from_c_args(interp, _call_object, "Pi",
            &pmc);
#line 435 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    { /* BEGIN PMETHOD BODY */
#line 1753 "./src/pmc/complex.pmc"

    PMC * const d = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));
    PMC     *e;
    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, d,  re / (re * re + im * im));
    SETATTR_Complex_im(interp, d, -im / (re * re + im * im));

    #line 2870 "./src/pmc/complex.pmc"
Parrot_pcc_invoke_method_from_c_args(interp, d, CONST_STRING_GEN(interp, "asinh"), "->P", &e);
#line 1763 "./src/pmc/complex.pmc"

    #line 260 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
#line 267 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    Parrot_pcc_fill_returns_from_c_args(interp, _call_object, "P",
            (PMC*)e);
    return;
    /*END RETURN PMC *e */
    }
#line 1764 "./src/pmc/complex.pmc"

#line 442 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    } /* END PMETHOD BODY */
    } /* END PARAMS SCOPE */
    return;
#line 2890 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_asech(PARROT_INTERP, PMC *pmc)
{
#line 423 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    PMC * const _ctx         = CURRENT_CONTEXT(interp);
    PMC * const _ccont       = Parrot_pcc_get_continuation(interp, _ctx);
    PMC * const _call_object = Parrot_pcc_get_signature(interp, _ctx);

    Parrot_pcc_set_signature(interp, _ctx, NULL);

    { /* BEGIN PARMS SCOPE */
#line 2903 "./src/pmc/complex.pmc"
PMC* pmc;

#line 431 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
        Parrot_pcc_fill_params_from_c_args(interp, _call_object, "Pi",
            &pmc);
#line 435 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    { /* BEGIN PMETHOD BODY */
#line 1767 "./src/pmc/complex.pmc"

    PMC * const d = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));
    PMC     *e;
    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, d,  re / (re * re + im * im));
    SETATTR_Complex_im(interp, d, -im / (re * re + im * im));

    #line 2922 "./src/pmc/complex.pmc"
Parrot_pcc_invoke_method_from_c_args(interp, d, CONST_STRING_GEN(interp, "acosh"), "->P", &e);
#line 1777 "./src/pmc/complex.pmc"

    #line 260 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
#line 267 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    Parrot_pcc_fill_returns_from_c_args(interp, _call_object, "P",
            (PMC*)e);
    return;
    /*END RETURN PMC *e */
    }
#line 1778 "./src/pmc/complex.pmc"

#line 442 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    } /* END PMETHOD BODY */
    } /* END PARAMS SCOPE */
    return;
#line 2942 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_pow(PARROT_INTERP, PMC *pmc)
{
#line 423 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    PMC * const _ctx         = CURRENT_CONTEXT(interp);
    PMC * const _ccont       = Parrot_pcc_get_continuation(interp, _ctx);
    PMC * const _call_object = Parrot_pcc_get_signature(interp, _ctx);

    Parrot_pcc_set_signature(interp, _ctx, NULL);

    { /* BEGIN PARMS SCOPE */
#line 2955 "./src/pmc/complex.pmc"
PMC* pmc;
PMC* value;

#line 431 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
        Parrot_pcc_fill_params_from_c_args(interp, _call_object, "PiP",
            &pmc, &value);
#line 435 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    { /* BEGIN PMETHOD BODY */
#line 1803 "./src/pmc/complex.pmc"

    PMC *l = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));
    PMC *log;
    PMC *dest = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));

    Parrot_pcc_invoke_method_from_c_args(interp, pmc, CONST_STRING(interp, "ln"),
                "->P", &log);

    l = VTABLE_multiply(interp, log, value, l);

    Parrot_pcc_invoke_method_from_c_args(interp, l, CONST_STRING(interp, "exp"),
                "->P", &dest);
    #line 260 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *dest */
#line 267 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    Parrot_pcc_fill_returns_from_c_args(interp, _call_object, "P",
            (PMC*)dest);
    return;
    /*END RETURN PMC *dest */
    }
#line 1815 "./src/pmc/complex.pmc"

#line 442 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    } /* END PMETHOD BODY */
    } /* END PARAMS SCOPE */
    return;
#line 2993 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_sqrt(PARROT_INTERP, PMC *pmc)
{
#line 423 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    PMC * const _ctx         = CURRENT_CONTEXT(interp);
    PMC * const _ccont       = Parrot_pcc_get_continuation(interp, _ctx);
    PMC * const _call_object = Parrot_pcc_get_signature(interp, _ctx);

    Parrot_pcc_set_signature(interp, _ctx, NULL);

    { /* BEGIN PARMS SCOPE */
#line 3006 "./src/pmc/complex.pmc"
PMC* pmc;

#line 431 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
        Parrot_pcc_fill_params_from_c_args(interp, _call_object, "Pi",
            &pmc);
#line 435 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    { /* BEGIN PMETHOD BODY */
#line 1818 "./src/pmc/complex.pmc"

    PMC * const result = Parrot_pmc_new(interp, VTABLE_type(interp, pmc));
    const FLOATVAL absval = pmc->vtable->get_number(interp, pmc);
    FLOATVAL sx, sy, rx, ry;
    GETATTR_Complex_re(interp, pmc, sx);
    GETATTR_Complex_im(interp, pmc, sy);

    rx = sqrt((absval + sx) / 2);
    ry = sqrt((absval - sx) / 2);
    if (sy < 0)
        ry = -ry;
    SETATTR_Complex_re(interp, result, rx);
    SETATTR_Complex_im(interp, result, ry);
    #line 260 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *result */
#line 267 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    Parrot_pcc_fill_returns_from_c_args(interp, _call_object, "P",
            (PMC*)result);
    return;
    /*END RETURN PMC *result */
    }
#line 1831 "./src/pmc/complex.pmc"

#line 442 "/builddir/build/BUILD/parrot-2.2.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    } /* END PMETHOD BODY */
    } /* END PARAMS SCOPE */
    return;
#line 3044 "./src/pmc/complex.c"
}
#include "pmc/pmc_default.h"

PARROT_EXPORT
VTABLE *Parrot_Complex_update_vtable(VTABLE *vt) {
    vt->absolute = Parrot_Complex_absolute;
    vt->add = Parrot_Complex_add;
    vt->add_float = Parrot_Complex_add_float;
    vt->add_int = Parrot_Complex_add_int;
    vt->clone = Parrot_Complex_clone;
    vt->divide = Parrot_Complex_divide;
    vt->divide_float = Parrot_Complex_divide_float;
    vt->divide_int = Parrot_Complex_divide_int;
    vt->freeze = Parrot_Complex_freeze;
    vt->get_bool = Parrot_Complex_get_bool;
    vt->get_integer = Parrot_Complex_get_integer;
    vt->get_integer_keyed = Parrot_Complex_get_integer_keyed;
    vt->get_integer_keyed_str = Parrot_Complex_get_integer_keyed_str;
    vt->get_number = Parrot_Complex_get_number;
    vt->get_number_keyed = Parrot_Complex_get_number_keyed;
    vt->get_number_keyed_int = Parrot_Complex_get_number_keyed_int;
    vt->get_number_keyed_str = Parrot_Complex_get_number_keyed_str;
    vt->get_pmc_keyed = Parrot_Complex_get_pmc_keyed;
    vt->get_pmc_keyed_int = Parrot_Complex_get_pmc_keyed_int;
    vt->get_pmc_keyed_str = Parrot_Complex_get_pmc_keyed_str;
    vt->get_string = Parrot_Complex_get_string;
    vt->i_absolute = Parrot_Complex_i_absolute;
    vt->i_add = Parrot_Complex_i_add;
    vt->i_add_float = Parrot_Complex_i_add_float;
    vt->i_add_int = Parrot_Complex_i_add_int;
    vt->i_divide = Parrot_Complex_i_divide;
    vt->i_divide_float = Parrot_Complex_i_divide_float;
    vt->i_divide_int = Parrot_Complex_i_divide_int;
    vt->i_multiply = Parrot_Complex_i_multiply;
    vt->i_multiply_float = Parrot_Complex_i_multiply_float;
    vt->i_multiply_int = Parrot_Complex_i_multiply_int;
    vt->i_neg = Parrot_Complex_i_neg;
    vt->i_subtract = Parrot_Complex_i_subtract;
    vt->i_subtract_float = Parrot_Complex_i_subtract_float;
    vt->i_subtract_int = Parrot_Complex_i_subtract_int;
    vt->init = Parrot_Complex_init;
    vt->init_pmc = Parrot_Complex_init_pmc;
    vt->is_equal = Parrot_Complex_is_equal;
    vt->multiply = Parrot_Complex_multiply;
    vt->multiply_float = Parrot_Complex_multiply_float;
    vt->multiply_int = Parrot_Complex_multiply_int;
    vt->neg = Parrot_Complex_neg;
    vt->set_integer_keyed = Parrot_Complex_set_integer_keyed;
    vt->set_integer_keyed_str = Parrot_Complex_set_integer_keyed_str;
    vt->set_integer_native = Parrot_Complex_set_integer_native;
    vt->set_number_keyed = Parrot_Complex_set_number_keyed;
    vt->set_number_keyed_int = Parrot_Complex_set_number_keyed_int;
    vt->set_number_keyed_str = Parrot_Complex_set_number_keyed_str;
    vt->set_number_native = Parrot_Complex_set_number_native;
    vt->set_pmc = Parrot_Complex_set_pmc;
    vt->set_pmc_keyed = Parrot_Complex_set_pmc_keyed;
    vt->set_pmc_keyed_str = Parrot_Complex_set_pmc_keyed_str;
    vt->set_string_native = Parrot_Complex_set_string_native;
    vt->subtract = Parrot_Complex_subtract;
    vt->subtract_float = Parrot_Complex_subtract_float;
    vt->subtract_int = Parrot_Complex_subtract_int;
    vt->thaw = Parrot_Complex_thaw;
    vt->attr_size = sizeof(Parrot_Complex_attributes);

    return vt;
}


PARROT_EXPORT
VTABLE *Parrot_Complex_ro_update_vtable(ARGMOD(VTABLE *vt)) {
    vt->absolute = Parrot_Complex_absolute;
    vt->add = Parrot_Complex_add;
    vt->add_float = Parrot_Complex_add_float;
    vt->add_int = Parrot_Complex_add_int;
    vt->clone = Parrot_Complex_clone;
    vt->divide = Parrot_Complex_divide;
    vt->divide_float = Parrot_Complex_divide_float;
    vt->divide_int = Parrot_Complex_divide_int;
    vt->freeze = Parrot_Complex_freeze;
    vt->get_bool = Parrot_Complex_get_bool;
    vt->get_integer = Parrot_Complex_get_integer;
    vt->get_integer_keyed = Parrot_Complex_get_integer_keyed;
    vt->get_integer_keyed_str = Parrot_Complex_get_integer_keyed_str;
    vt->get_number = Parrot_Complex_get_number;
    vt->get_number_keyed = Parrot_Complex_get_number_keyed;
    vt->get_number_keyed_int = Parrot_Complex_get_number_keyed_int;
    vt->get_number_keyed_str = Parrot_Complex_get_number_keyed_str;
    vt->get_pmc_keyed = Parrot_Complex_get_pmc_keyed;
    vt->get_pmc_keyed_int = Parrot_Complex_get_pmc_keyed_int;
    vt->get_pmc_keyed_str = Parrot_Complex_get_pmc_keyed_str;
    vt->get_string = Parrot_Complex_get_string;
    vt->init = Parrot_Complex_init;
    vt->init_pmc = Parrot_Complex_init_pmc;
    vt->is_equal = Parrot_Complex_is_equal;
    vt->multiply = Parrot_Complex_multiply;
    vt->multiply_float = Parrot_Complex_multiply_float;
    vt->multiply_int = Parrot_Complex_multiply_int;
    vt->neg = Parrot_Complex_neg;
    vt->subtract = Parrot_Complex_subtract;
    vt->subtract_float = Parrot_Complex_subtract_float;
    vt->subtract_int = Parrot_Complex_subtract_int;
    vt->attr_size = sizeof(Parrot_Complex_attributes);

    return vt;
}

PARROT_EXPORT
PARROT_CANNOT_RETURN_NULL
PARROT_WARN_UNUSED_RESULT
VTABLE* Parrot_Complex_get_vtable(PARROT_INTERP) {
    VTABLE *vt;
    vt = Parrot_default_get_vtable(interp);
    Parrot_Complex_update_vtable(vt);

    return vt;
}

PARROT_EXPORT
PARROT_CANNOT_RETURN_NULL
PARROT_WARN_UNUSED_RESULT
VTABLE* Parrot_Complex_ro_get_vtable(PARROT_INTERP) {
    VTABLE *vt;
    vt = Parrot_default_ro_get_vtable(interp);
    Parrot_Complex_ro_update_vtable(vt);

    return vt;
}

PARROT_EXPORT
PARROT_CANNOT_RETURN_NULL
PARROT_WARN_UNUSED_RESULT
PMC* Parrot_Complex_get_mro(PARROT_INTERP, PMC* mro) {
    if (PMC_IS_NULL(mro)) {
        mro = Parrot_pmc_new(interp, enum_class_ResizableStringArray);
    }

    VTABLE_unshift_string(interp, mro,
        string_make(interp, "Complex", 7, NULL, 0));
    return mro;
}

PARROT_EXPORT
PARROT_CANNOT_RETURN_NULL
PARROT_WARN_UNUSED_RESULT
Hash* Parrot_Complex_get_isa(PARROT_INTERP, Hash* isa) {
    if (isa == NULL) {
        isa = parrot_new_hash(interp);
    }

    parrot_hash_put(interp, isa, (void *)(CONST_STRING_GEN(interp, "Complex")), PMCNULL);
    return isa;
}

void
Parrot_Complex_class_init(PARROT_INTERP, int entry, int pass)
{
    static const char attr_defs [] =
        "Fre Fim ";
    if (pass == 0) {
        VTABLE * const vt  = Parrot_Complex_get_vtable(interp);
        vt->base_type      = enum_class_Complex;
        vt->flags          = 0|VTABLE_HAS_READONLY_FLAG;
        vt->attribute_defs = attr_defs;
        interp->vtables[entry] = vt;

        vt->whoami       = CONST_STRING_GEN(interp, "Complex");
        vt->provides_str = CONST_STRING_GEN(interp, "scalar");
        vt->isa_hash     = NULL;
        {
            VTABLE                   *vt_ro;
            vt_ro                 = Parrot_Complex_ro_get_vtable(interp);
            vt_ro->base_type      = enum_class_Complex;
            vt_ro->flags          = 0|VTABLE_IS_READONLY_FLAG;

            vt_ro->attribute_defs = attr_defs;

            vt_ro->base_type           = entry;
            vt_ro->whoami              = vt->whoami;
            vt_ro->provides_str        = vt->provides_str;
            vt->ro_variant_vtable      = vt_ro;
            vt_ro->ro_variant_vtable = vt;
            vt_ro->isa_hash            = vt->isa_hash;
        }

    }
    else { /* pass */
        {
            VTABLE * const vt  = interp->vtables[entry];

            vt->mro = Parrot_Complex_get_mro(interp, PMCNULL);

            if (vt->ro_variant_vtable)
                vt->ro_variant_vtable->mro = vt->mro;
        }

        /* set up MRO and _namespace */
        Parrot_pmc_create_mro(interp, entry);
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_ln), CONST_STRING_GEN(interp, "ln"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_exp), CONST_STRING_GEN(interp, "exp"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_sin), CONST_STRING_GEN(interp, "sin"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_cos), CONST_STRING_GEN(interp, "cos"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_tan), CONST_STRING_GEN(interp, "tan"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_cot), CONST_STRING_GEN(interp, "cot"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_sec), CONST_STRING_GEN(interp, "sec"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_csc), CONST_STRING_GEN(interp, "csc"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_asin), CONST_STRING_GEN(interp, "asin"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_acos), CONST_STRING_GEN(interp, "acos"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_atan), CONST_STRING_GEN(interp, "atan"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_acot), CONST_STRING_GEN(interp, "acot"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_acsc), CONST_STRING_GEN(interp, "acsc"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_asec), CONST_STRING_GEN(interp, "asec"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_sinh), CONST_STRING_GEN(interp, "sinh"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_cosh), CONST_STRING_GEN(interp, "cosh"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_tanh), CONST_STRING_GEN(interp, "tanh"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_coth), CONST_STRING_GEN(interp, "coth"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_csch), CONST_STRING_GEN(interp, "csch"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_sech), CONST_STRING_GEN(interp, "sech"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_asinh), CONST_STRING_GEN(interp, "asinh"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_acosh), CONST_STRING_GEN(interp, "acosh"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_atanh), CONST_STRING_GEN(interp, "atanh"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_acoth), CONST_STRING_GEN(interp, "acoth"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_acsch), CONST_STRING_GEN(interp, "acsch"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_asech), CONST_STRING_GEN(interp, "asech"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_pow), CONST_STRING_GEN(interp, "pow"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_sqrt), CONST_STRING_GEN(interp, "sqrt"));
        {
        STRING * const mfl_0 = CONST_STRING_GEN(interp, "add");
        STRING * const mfl_1 = CONST_STRING_GEN(interp, "PJPPP");
        STRING * const mfl_2 = CONST_STRING_GEN(interp, "Complex,Complex,PMC");
        STRING * const mfl_3 = CONST_STRING_GEN(interp, "Complex");
        STRING * const mfl_4 = CONST_STRING_GEN(interp, "Complex,DEFAULT,PMC");
        STRING * const mfl_5 = CONST_STRING_GEN(interp, "i_add");
        STRING * const mfl_6 = CONST_STRING_GEN(interp, "vJPP");
        STRING * const mfl_7 = CONST_STRING_GEN(interp, "Complex,Complex");
        STRING * const mfl_8 = CONST_STRING_GEN(interp, "Complex,DEFAULT");
        STRING * const mfl_9 = CONST_STRING_GEN(interp, "subtract");
        STRING * const mfl_10 = CONST_STRING_GEN(interp, "i_subtract");
        STRING * const mfl_11 = CONST_STRING_GEN(interp, "multiply");
        STRING * const mfl_12 = CONST_STRING_GEN(interp, "i_multiply");
        STRING * const mfl_13 = CONST_STRING_GEN(interp, "divide");
        STRING * const mfl_14 = CONST_STRING_GEN(interp, "i_divide");
        STRING * const mfl_15 = CONST_STRING_GEN(interp, "is_equal");
        STRING * const mfl_16 = CONST_STRING_GEN(interp, "IJPP");

             const  multi_func_list _temp_multi_func_list[] = {
                        { mfl_0,
          mfl_1,
          mfl_2,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_add_Complex_PMC }
,
        { mfl_0,
          mfl_1,
          mfl_4,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_add_DEFAULT_PMC }
,
        { mfl_5,
          mfl_6,
          mfl_7,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_i_add_Complex }
,
        { mfl_5,
          mfl_6,
          mfl_8,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_i_add_DEFAULT }
,
        { mfl_9,
          mfl_1,
          mfl_2,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_subtract_Complex_PMC }
,
        { mfl_9,
          mfl_1,
          mfl_4,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_subtract_DEFAULT_PMC }
,
        { mfl_10,
          mfl_6,
          mfl_7,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_i_subtract_Complex }
,
        { mfl_10,
          mfl_6,
          mfl_8,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_i_subtract_DEFAULT }
,
        { mfl_11,
          mfl_1,
          mfl_2,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_multiply_Complex_PMC }
,
        { mfl_11,
          mfl_1,
          mfl_4,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_multiply_DEFAULT_PMC }
,
        { mfl_12,
          mfl_6,
          mfl_7,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_i_multiply_Complex }
,
        { mfl_12,
          mfl_6,
          mfl_8,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_i_multiply_DEFAULT }
,
        { mfl_13,
          mfl_1,
          mfl_2,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_divide_Complex_PMC }
,
        { mfl_13,
          mfl_1,
          mfl_4,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_divide_DEFAULT_PMC }
,
        { mfl_14,
          mfl_6,
          mfl_7,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_i_divide_Complex }
,
        { mfl_14,
          mfl_6,
          mfl_8,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_i_divide_DEFAULT }
,
        { mfl_15,
          mfl_16,
          mfl_7,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_is_equal_Complex }
,
        { mfl_15,
          mfl_16,
          mfl_8,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_is_equal_DEFAULT }

            };
#define N_MULTI_LIST (sizeof(_temp_multi_func_list)/sizeof(_temp_multi_func_list[0]))
            Parrot_mmd_add_multi_list_from_c_args(interp,
                _temp_multi_func_list, N_MULTI_LIST);
        }
    } /* pass */
} /* Parrot_Complex_class_init */
#line 1833 "./src/pmc/complex.pmc"


/*

=back

=cut

*/

/*
 * Local variables:
 *   c-file-style: "parrot"
 * End:
 * vim: expandtab shiftwidth=4:
 */
